<?php
namespace Teknisa\SUP\Modules\Cotacao\Modules\AnaliseCotacoes\Controllers;

use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;

class Sup03400ConsultaDeMelhoresPrecosController extends \Zeedhi\Framework\Controller\Report {


    protected $entityManager;
    protected $qr2Strategy;
    protected $newCode;
    protected $environment;
    protected $connection;
    protected $Sup03400ConsultaDeMelhoresPrecosService;
    protected $reportMapping;

    public function __construct($reportService, $entityManager, $qr2Strategy, $newCode, $connection, $environment, $Sup03400ConsultaDeMelhoresPrecosService) {
        $this->reportService  = $reportService;
        $this->entityManager  = $entityManager;
        $this->qr2Strategy    = $qr2Strategy;
        $this->newCode        = $newCode;
        $this->connection     = $connection;
        $this->environment    = $environment;
        $this->Sup03400ConsultaDeMelhoresPrecosService = $Sup03400ConsultaDeMelhoresPrecosService;
    }
    
    public function findSubgrupo(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $cdfilial = $this->environment->getCdFilial();
            $dataSet = $this->Sup03400ConsultaDeMelhoresPrecosService->getSubgrupos($cdfilial);
            $response->addDataSet(new DataSet('subgrupprod_filial', $dataSet));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
    }
    
    public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        $cdfilial = $this->environment->getCdFilial();
        $row["P_CDFILIAL"] = $this->Sup03400ConsultaDeMelhoresPrecosService->getCdFilial($cdfilial);
        $this->initializeReportMapping();
        
        $NRLISTA = $this->newCode->getCodeByTableName('NRLISTA', $this->environment->getNrorgTrab(), 1, 10, false);
        $this->insertLista($NRLISTA);
        
        $row['__report_name'] = $row['REPORT_NAME'];
        $this->qr2Strategy->setQr2Type($row['QR2TYPE']);
    }
    
    public function insertLista($nrlista) {
        $params = array(
            "NRLISTA" => $nrlista,
            "CDLISTA" => $this->environment->getCdOperador()
        );
        $sql = "INSERT INTO LISTA (NRLISTA, CDLISTA) VALUES (:NRLISTA, :CDLISTA)";
        $query = $this->connection->prepare($sql);
        $query->execute($params);
    }
    
    public function initializeReportMapping() {
        $params = array (
            'strategy' => 'QR2',
            'parameterMapping' =>
                array (
                    "L_CDFORNECED"   => "L_CDFORNECED",
                    "L_CDOPERADOR"   => "L_CDOPERADOR",
                    "L_CDREGIAO"     => "L_CDREGIAO",
                    "L_CDREGIAO1"    => "L_CDREGIAO1",
                    "L_CDREGIAO2"    => "L_CDREGIAO2",
                    "L_CDREGIAO3"    => "L_CDREGIAO3",
                    "L_CDREGIAO4"    => "L_CDREGIAO4",
                    "P_CDFILIAL"     => "P_CDFILIAL",
                    "P_CDPRODFIN"    => "P_CDPRODFIN",
                    "P_CDPRODFINAL"  => "P_CDPRODFINAL",
                    "P_CDPRODINI"    => "P_CDPRODINI",
                    "P_DTFIN"        => "P_DTFIN",
                    "P_DTINI"        => "P_DTINI",
                    "P_SUBGRUPO"     => "P_SUBGRUPO",
                    "P_TCDPRODUTO"   => "P_TCDPRODUTO",
                    "P_TPROD"        => "P_TPROD",
                    "P_VALIDAS"      => "P_VALIDAS",
                    "P_VENCIDAS"     => "P_VENCIDAS"
                ),
        );
        
        $this->reportMapping = array (
            'SUP03400' => $params,
            'SUP03401' => $params,
            'SUP03402' => $params,
            'SUP03403' => $params,
            'SUP03404' => $params,
            'SUP03405' => $params,
            'SUP03406' => $params,
            'SUP03407' => $params,
            'SUP03408' => $params,
            'SUP03409' => $params,
            'SUP0340A' => $params,
            'SUP0340B' => $params,
            'SUP0340F' => $params,
            'SUP0340G' => $params,
            'SUP0340H' => $params
        );
    }
}