<?php

namespace Teknisa\SUP\Modules\Cotacao\Modules\PlanilhaCotacao\Services;

use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DTO;
use Teknisa\SUP\Util\SupUtilities;

class Sup05200PlanilhaDeCotacaoService
{
    protected $entityManager;
    protected $environment;
    protected $connection;
    
    public function __construct($entityManager, $environment, $connection, SupUtilities $supUtilities)
    {
        $this->entityManager = $entityManager;
        $this->environment   = $environment;
        $this->connection    = $connection;
        $this->supUtilities  = $supUtilities;
    }
    
    /********************
     *   TRANSACTIONS   *
     ********************/
    
    public function beginTransaction()
    {
        $this->entityManager->beginTransaction();
    }
    
    public function commitTransaction()
    {
        $this->entityManager->commit();
    }
    
    public function rollbackTransaction()
    {
        $this->entityManager->rollback();
    }
    
    public function getcentralCompras($cdfilial = null) 
    {
        return $this->supUtilities->getCentralCompras($cdfilial);
    }
    
    /********************
     *      SELECTS     *
     * ******************/
     
    public function buscaData($central)
    {   
        $query = "SELECT COLETAPR.DTINIVALCOL, COLETAPR.DTFINVALCOL, COLETAPR.NRORG 
                    FROM COLETAPR, (SELECT MAX(COLETAPR.NRCOLEPRECO) AS NRCOLEPRECO 
                                      FROM COLETAPR INNER JOIN ITEMCOLE ON COLETAPR.CDFILIAL    = ITEMCOLE.CDFILIAL
                                                                       AND COLETAPR.NRCOLEPRECO = ITEMCOLE.NRCOLEPRECO
                                                                       AND COLETAPR.NRORG       = ITEMCOLE.NRORG
                                     WHERE COLETAPR.CDFILIAL = :P_CDFILIAL 
                                       AND COLETAPR.CDOPERADOR = :P_CDOPERADOR ) ULTCOT 
                   WHERE COLETAPR.NRCOLEPRECO   = ULTCOT.NRCOLEPRECO
                     AND COLETAPR.CDFILIAL      = :P_CDFILIAL 
                     AND COLETAPR.NRORG         = :P_NRORG";
        $params = [
            'P_CDOPERADOR'  => $this->environment->getCdoperador(),
            'P_CDFILIAL'    => $central['CDFILCENTCMP'],
            'P_NRORG'       => $this->environment->getNrorg()
        ];
        return $this->supUtilities->crud($query, $params);
    }
    
    public function buscaParametros($central)
    {    
        $query = "SELECT CDFILIAL , IDJUSSOLEXT , IDBLOQALTNEC, IDAUDALTSOL, 
                         IDULTIDTULTI, IDULTCOMPSOL, IDULTDESTSOL, CDGRUPOPER, 
                         IDULTIDTVALQ, IDVALDTAF , IDBLOQUEAAF , IDMARSEMQUA, 
                         IDTIPOAPUR, IDUTLPROCMP, NRCOTESCAUTO, IDVISCOTPLA, 
                         IDUMPRODCMP, CDMARCA, IDPROSEMMAR, CDGRUPPROD, 
                         IDPERMREPLICREG, IDPERMREPLICFOR, DSDIRLOGAUTF, 
                         NVL(IDUTISTATCOT, 'N') AS IDUTISTATCOT, 
                         NVL(IDAUDALTCOT, 'N') AS IDAUDALTCOT, IDCONFREMANEJAF, 
                         IDCOTCONCLUI, NVL(IDUTINDICECOL, 'N') IDUTINDICECOL 
                    FROM PARASUPRI
                   WHERE CDFILIAL = :P_CDFILCENT
                     AND NRORG    = :P_NRORG";
        $params = [
            'P_CDFILCENT'   => $central['CDFILCENTCMP'],
            'P_NRORG'       => $this->environment->getNrorg()
        ];
        return $this->supUtilities->crud($query, $params);
    }
    
    public function verifyIndiceFinanceiro($row)
    {
        $query = "SELECT NRCOLEPRECO
                    FROM COLETAPR
                   WHERE CDFILIAL    = :P_CDFILIAL
                     AND NRCOLEPRECO = :P_NRCOLEPRECO
                     AND NRORG       = :NRORG
                     AND CDINDICECOL IS NULL";
        $params = [
            'P_CDFILIAL'    => $row['CDFILIAL'],
            'P_NRCOLEPRECO' => $row['NRCOLEPRECO'],
            'NRORG'         => $this->environment->getNrorg()
        ];
        return $this->supUtilities->crud($query, $params);
        
    }
    
    public function verificaOperadorComprador()
    {    
        $query = "SELECT CDOPERADOR 
                    FROM COMPRADOR 
                   WHERE CDOPERADOR = :P_CDOPERADOR 
                     AND NRORG      = :P_NRORG";
        $params = [
            'P_CDOPERADOR'  => $this->environment->getCdoperador(),
            'P_NRORG'       => $this->environment->getNrorg()
        ];
        return $this->supUtilities->crud($query, $params);
    }
    
    public function operadoresRelacionados($row){    
        $query = "SELECT CDOPERADOR, IDEXSENHCMPOPER, IDCOTCMPOPER, IDAPURCMPOPER, IDAFCMPOPER 
                    FROM COMPROPER 
                   WHERE CDCOMPRADOR = :P_CDCOMPRADOR
                     AND NRORG       = :P_NRORG";
        $params = [
            'P_CDCOMPRADOR' => $row['CDOPERADOR'],
            'P_NRORG'       => $this->environment->getNrorg()
        ];
        return $this->supUtilities->crud($query, $params);
    }
    
    public function senhaGerente($row){    
        $query = "SELECT CDOPERADOR, NMOPERADOR, CDSENHOPER, CDSENHAOPERWEB FROM OPERADOR WHERE CDOPERADOR = :P_CDOPERADOR AND NRORG = :P_NRORG";
        $params = [
            'P_CDOPERADOR'  => $row['CDGERENTE'],
            'P_NRORG'       => $this->environment->getNrorg()
        ];
        return $this->supUtilities->crud($query, $params);
    }
    
    
    
    public function validaSituacaoColeta($row){    
        $query = "SELECT NVL(COLETAPR.IDSITUCOLETA,'C') AS IDSITUCOLETA FROM COLETAPR, OPERADOR WHERE COLETAPR.NRCOLEPRECO = :P_NRCOLEPRECO AND OPERADOR.CDOPERADOR = :P_CDOPERADOR AND COLETAPR.NRORG = OPERADOR.NRORG AND OPERADOR.NRORG = :P_NRORG";
        $params = [
            'P_CDOPERADOR'  => $row['CDOPERADOR'],
            'P_NRCOLEPRECO' => $row['NRCOLEPRECO'],
            'P_NRORG'       => $this->environment->getNrorg()
        ];
        return $this->supUtilities->crud($query, $params);
    }
    
    public function validaCadastroComprador($row){    
        $query = "SELECT COMPRADOR.CDOPERADOR, OPERADOR.NMOPERADOR FROM COMPRADOR, OPERADOR WHERE (COMPRADOR.CDOPERADOR = :P_CDOPERADOR) AND (COMPRADOR.CDOPERADOR = OPERADOR.CDOPERADOR) AND (COMPRADOR.NRORG = OPERADOR.NRORG) AND (OPERADOR.NRORG = :P_NRORG) ORDER BY COMPRADOR.CDOPERADOR";
        $params = [
            'P_CDOPERADOR'  => $row['CDCOMPRADOR'],
            'P_NRORG'       => $this->environment->getNrorg()
        ];
        return $this->supUtilities->crud($query, $params);
    }
    
    public function listaPreco($row){    
        $query = "SELECT COUNT(*) AS CONTA FROM ITEMCOLE WHERE CDFILIAL = :P_CDFILIAL AND NRCOLEPRECO = :P_NRCOLEPRECO AND CDREGIAO = NVL(:P_CDREGIAO, CDREGIAO) AND CDFORNECED = NVL(:P_CDFORNECED, CDFORNECED) AND CDPRODUTO = NVL(:P_CDPRODUTO, CDPRODUTO) AND CDMARCA = NVL(:P_CDMARCA, CDMARCA) AND IDITMLISPREC = 'S' AND NRORG = :P_NRORG";
        $params = [
            'P_CDFILIAL'    => $row['CDFILIAL'],
            'P_NRCOLEPRECO' => $row['NRCOLEPRECO'],
            'P_CDREGIAO'    => null,
            'P_CDFORNECED'  => null,
            'P_CDPRODUTO'   => null,
            'P_CDMARCA'     => null,
            'P_NRORG'       => null
        ];
        return $this->supUtilities->crud($query, $params);
    }
    
    public function getParaSupri($row){    
        $query = "SELECT IDUTLPROCMP, IDPROSEMMAR, CDMARCA, IDUTISTATCOT, NVL(IDUTINDICECOL, 'N') AS IDUTINDICECOL FROM PARASUPRI WHERE CDFILIAL = :P_CDFILIAL AND NRORG = :P_NRORG";
        $params = [
            'P_CDFILIAL'    => $row['CDFILIAL'],
            'P_NRORG'       => $this->environment->getNrorg()
        ];
        return $this->supUtilities->crud($query, $params);
    }
    
    public function verificaRegiaoExiste($row){    
        $query = "SELECT DISTINCT :P_NRCOLEPRECO AS NRCOLEPRECO, ITEMCOLE.CDREGIAO || FORNECEDOR.CDFORNECED AS CHAVE, ITEMCOLE.CDREGIAO, ITEMCOLE.NMREGIAO, ITEMCOLE.CDREGIAO || ' ' || ITEMCOLE.NMREGIAO AS REGIAO, FORNECEDOR.CDFORNECED, FORNECEDOR.IDTPIJURFORN, FORNECEDOR.NRINSJURFORN, FORNECEDOR.NRORG, RTRIM(FORNECEDOR.NMFANTFORN) AS NMFANTFORN, RTRIM(FORNECEDOR.NMFANTFORN) AS NMRAZSOCFORN FROM FORNECEDOR, ( SELECT DISTINCT ITEMCOLE.CDFORNECED, ITEMCOLE.CDREGIAO, REGIAO.NMREGIAO, ITEMCOLE.NRORG FROM ITEMCOLE, REGIAO WHERE ITEMCOLE.CDFILIAL =:P_CDFILIAL AND ITEMCOLE.NRCOLEPRECO =:P_NRCOLEPRECO AND REGIAO.CDREGIAO = ITEMCOLE.CDREGIAO AND ITEMCOLE.CDREGIAO = :P_CDREGIAO AND ITEMCOLE.NRORG =:P_NRORG ) ITEMCOLE WHERE ITEMCOLE.CDFORNECED = FORNECEDOR.CDFORNECED AND FORNECEDOR.NRORG = ITEMCOLE.NRORG ORDER BY ITEMCOLE.CDREGIAO, RTRIM(FORNECEDOR.NMFANTFORN)";
        $params = [
            'P_CDFILIAL'    => $row['CDFILIAL'],
            'P_NRCOLEPRECO' => $row['NRCOLEPRECO'],
            'P_CDREGIAO'    => $row['CDREGIAO'],
            'P_NRORG'       => $this->environment->getNrorg()
        ];
        return $this->supUtilities->crud($query, $params);
    }
    
    public function getFornecedoresCotacao($row){    
        $query = "SELECT DISTINCT FORNECEDOR.CDFORNECED, FORNECEDOR.NMFANTFORN, FORNECEDOR.NMRAZSOCFORN, FORNECEDOR.IDTPIJURFORN, FORNECEDOR.NRINSJURFORN FROM FORNECEDOR, ITEMCOLE WHERE FORNECEDOR.CDFORNECED = ITEMCOLE.CDFORNECED AND ITEMCOLE.CDFILIAL = :P_CDFILIAL AND ITEMCOLE.NRCOLEPRECO = :P_NRCOLEPRECO AND FORNECEDOR.NRORG = ITEMCOLE.NRORG AND ITEMCOLE.NRORG = :P_NRORG";
        $params = [
            'P_CDFILIAL'    => $row['CDFILIAL'],
            'P_NRCOLEPRECO' => $row['NRCOLEPRECO'],
            'P_NRORG'       => $this->environment->getNrorg()
        ];
        return $this->supUtilities->crud($query, $params);
    }
    
    public function verificaRegForn($params, $row){    
        $query = "SELECT CDREGIAO FROM REFOREG WHERE REFOREG.CDFORNECED =:P_CDFORNECED AND REFOREG.CDREGIAO =:P_CDREGIAO AND REFOREG.NRORG =:P_NRORG";
        $params = [
            'P_CDFORNECED'  => $params['CDFORNECED'],
            'P_CDREGIAO'    => $row['CDREGIAO'],
            'P_NRORG'       => $this->environment->getNrorg()
        ];
        return $this->supUtilities->crud($query, $params);
    }
    
    public function verificaSituacaoForn($params, $row){   
        $query = "SELECT FORNECEDOR.IDSITUCADA FROM FORNECEDOR, QUALIFICACAO WHERE FORNECEDOR.CDFORNECED = :P_CDFORNECED AND FORNECEDOR.NRORG = QUALIFICACAO.NRORG AND QUALIFICACAO.NRORG = :P_NRORG AND FORNECEDOR.IDSITUCADA = 'A' AND (FORNECEDOR.CDQUALIF IS NOT NULL AND FORNECEDOR.CDQUALIF = QUALIFICACAO.CDQUALIF AND ((QUALIFICACAO.IDPERCOTCMP = '1') OR (QUALIFICACAO.IDPERCOTCMP = '2' AND FORNECEDOR.IDFORNAUTO = 'S')) AND ( (:P_IDULTIDTVALQ = 'D' AND :P_DTCOLEPREC <= FORNECEDOR.DTVALQUALIF) OR (:P_IDULTIDTVALQ = 'M' AND :P_DTCOLEPREC <= LAST_DAY(FORNECEDOR.DTVALQUALIF)) ) ) UNION SELECT FORNECEDOR.IDSITUCADA FROM FORNECEDOR WHERE FORNECEDOR.CDFORNECED = :P_CDFORNECED AND FORNECEDOR.NRORG = :P_NRORG AND FORNECEDOR.CDQUALIF IS NULL AND FORNECEDOR.IDSITUCADA = 'A'";
        $params = [
            'P_CDFORNECED'  => $params['CDFORNECED'],
            'P_IDULTIDTVALQ'=> $row['IDULTIDTVALQ'],
            'P_DTCOLEPREC'  => $row['DTCOLEPREC'],
            'P_NRORG'       => $this->environment->getNrorg()
        ];
        return $this->supUtilities->crud($query, $params);
    }
    
    public function buscaProdutoCotacao($row){    
        $query = "SELECT DISTINCT PRODUTO.CDPRODUTO, PRODUTO.CDARVPROD, PRODUTO.NMPRODUTO, PRODUTO.SGUNIDADE FROM PRODUTO, ITEMCOLE WHERE ITEMCOLE.CDFILIAL =:P_CDFILIAL AND ITEMCOLE.NRCOLEPRECO =:P_NRCOLEPRECO AND ITEMCOLE.CDPRODUTO = PRODUTO.CDPRODUTO AND ITEMCOLE.NRORG = PRODUTO.NRORG AND PRODUTO.NRORG = :P_NRORG";
        $params = [
            'P_CDFILIAL'    => $row['CDFILIAL'],
            'P_NRCOLEPRECO' => $row['NRCOLEPRECO'],
            'P_NRORG'       => $this->environment->getNrorg()
        ];
        return $this->supUtilities->crud($query, $params);
    }
    
    public function getProdM($row, $CDFORNECED){    
        $query = "SELECT DISTINCT ITEMCOLE.CDPRODUTO, PRODUTO.NMPRODUTO, PRODUTO.SGUNIDADE, ITEMCOLE.CDMARCA, PRODUTO.CDARVPROD, COLETAPR.DTCOLEPREC FROM ITEMCOLE, PRODUTO, COLETAPR WHERE ITEMCOLE.CDPRODUTO = PRODUTO.CDPRODUTO AND ITEMCOLE.CDFILIAL = COLETAPR.CDFILIAL AND ITEMCOLE.NRCOLEPRECO = COLETAPR.NRCOLEPRECO AND ITEMCOLE.CDFILIAL = :P_CDFILIAL AND ITEMCOLE.NRCOLEPRECO = :P_NRCOLEPRECO AND ITEMCOLE.NRORG = PRODUTO.NRORG AND PRODUTO.NRORG = COLETAPR.NRORG AND COLETAPR.NRORG = :P_NRORG AND EXISTS (SELECT 1 FROM PRODFORNCOMP WHERE PRODFORNCOMP.CDFORNECED = :P_CDFORNECED AND PRODFORNCOMP.CDPRODCOMPL = ITEMCOLE.CDPRODUTO)";
        $params = [
            'P_CDFILIAL'    => $row['CDFILIAL'],
            'P_CDFORNECED'  => $CDFORNECED,
            'P_NRCOLEPRECO' => $row['NRCOLEPRECO'],
            'P_NRORG'       => $this->environment->getNrorg()
        ];
        return $this->supUtilities->crud($query, $params);
    }
    
    public function verifyProdForn($row, $prodM, $vdForn){ 
        $query = "SELECT FORNECEDOR.CDFORNECED FROM FORNECEDOR, PRODUTO WHERE ( PRODUTO.CDPRODUTO =:P_CDPRODINI ) AND (PRODUTO.CDPRODESTO IS NOT NULL) AND PRODUTO.NRORG = FORNECEDOR.NRORG AND FORNECEDOR.NRORG = :P_NRORG AND (FORNECEDOR.CDFORNECED = :P_CDFORNECED) AND NOT EXISTS (SELECT QFOPRODFORN.CDPRODUTO FROM QFOPRODFORN, PARASUPRI WHERE QFOPRODFORN.NRORG = PARASUPRI.NRORG AND PARASUPRI.NRORG = :P_NRORG AND PARASUPRI.CDFILIAL = :P_CDFILIAL AND (QFOPRODFORN.CDPRODUTO = PRODUTO.CDPRODUTO) AND (QFOPRODFORN.CDFORNECED = FORNECEDOR.CDFORNECED) AND ((QFOPRODFORN.IDFORNAPROV = 'N' AND QFOPRODFORN.IDFORNAUTOR = 'N') OR ((QFOPRODFORN.IDFORNAPROV = 'S') AND ((PARASUPRI.IDULTIDTVALQ = 'M' AND :P_DTCOL > LAST_DAY(QFOPRODFORN.DTVALPRFORN)) OR (PARASUPRI.IDULTIDTVALQ = 'D' AND :P_DTCOL > QFOPRODFORN.DTVALPRFORN) ) ) ) )";
        $params = [
            'P_CDFILIAL'    => $row['CDFILIAL'],
            'P_CDFORNECED'  => $vdForn['CDFORNECED'],
            'P_DTCOL'       => $prodM['DTCOLEPREC'],
            'P_CDPRODINI'   => $prodM['CDPRODUTO'],
            'P_NRORG'       => $this->environment->getNrorg()
        ];
        return $this->supUtilities->crud($query, $params);
    }
    
    public function getPrecoUnitario($row, $prodM, $vdForn){    
        $query = "SELECT VRPRECUNIITM, NVL(VRINDICEITMCOL, 0) VRINDICEITMCOL FROM ITEMCOLE WHERE CDFILIAL =:P_CDFILIAL AND NRCOLEPRECO = :P_NRCOLEPRECO AND CDREGIAO = :P_CDREGIAO AND CDFORNECED = :P_CDFORNECED AND CDPRODUTO =:P_CDPRODUTO AND CDMARCA =:P_CDMARCA AND NRORG = :P_NRORG";
        $params = [
            'P_CDFILIAL'    => $row['CDFILIAL'],
            'P_CDFORNECED'  => $row['CDFORNECED'],
            'P_CDREGIAO'    => $row['CDREGIAOPREC'],
            'P_CDPRODUTO'   => $prodM['CDPRODUTO'],
            'P_CDMARCA'     => $prodM['CDMARCA'],
            'P_NRCOLEPRECO' => $row['NRCOLEPRECO'],
            'P_NRORG'       => $this->environment->getNrorg()
        ];
        return $this->supUtilities->crud($query, $params);
    }
    
    public function getListaPreco($row){    
        $query = "SELECT NRSEQPRELIST, IDPRECALT, IDITVALDIFLISTA FROM ( SELECT MAX(NRSEQPRELIST) NRSEQPRELIST, MAX(IDPRECALT) IDPRECALT, MAX(IDITVALDIFLISTA) IDITVALDIFLISTA FROM ITEMCOLE WHERE ITEMCOLE.CDFILIAL = :P_CDFILIAL AND ITEMCOLE.NRCOLEPRECO = :P_NRCOLEPRECO AND ITEMCOLE.NRORG = :P_NRORG AND ITEMCOLE.NRSEQPRELIST IS NOT NULL AND ITEMCOLE.IDITMLISPREC = 'S')";
        $params = [
            'P_CDFILIAL'    => $row['CDFILIAL'],
            'P_NRCOLEPRECO' => $row['NRCOLEPRECO'],
            'P_NRORG'       => $this->environment->getNrorg()
        ];
        return $this->supUtilities->crud($query, $params);
    }
    
    public function getEmitenteAF($params)
    {    
        try {
            $query = "SELECT EMITENTEAF.CDEMPRESA FROM PRODUTO, EMITENTEAF WHERE EMITENTEAF.CDFILIAL = :P_CDFILIAL AND EMITENTEAF.CDPRODUTO = PRODUTO.CDPRODUTO AND :P_CDPRODUTO LIKE PRODUTO.CDARVPROD || '%' AND EMITENTEAF.NRORG = PRODUTO .NRORG AND PRODUTO.NRORG = :P_NRORG";
            
            $params = [
                'P_CDFILIAL' => $params['CDFILIAL'],
                'P_CDPRODUTO' => $params['CDPRODUTO'],
                'P_NRORG' => $this->environment->getNrorg()
            ];
            
            return $this->supUtilities->crud($query, $params);
        } catch (Exception $e) {
            throw new \Exception('Erro ao buscar dados do emitente!');
        }
    }
    
    public function getEmitentePadrao($params)
    {
        try {
            $query = "SELECT PARCMPSUP.CDEMPRESA FROM PARCMPSUP WHERE PARCMPSUP.CDFILIAL =:P_CDFILIAL AND PARCMPSUP.NRORG = :P_NRORG";
            
            $params = [
                'P_CDFILIAL' => $params['CDFILIAL'],
                'P_NRORG' => $this->environment->getNrorg()
            ];
            
            return $this->supUtilities->crud($query, $params);
        } catch (Exception $e) {
            throw new \Exception('Erro ao buscar dados do emitente padrão!');
        }
    }
    
    public function getEmpresaFili($params)
    {
        try {
            $query = "SELECT CDEMPRESA FROM FILIAL WHERE FILIAL.CDFILIAL = :P_CDFILIAL AND FILIAL.NRORG = :P_NRORG";
            
            $params = [
                'P_CDFILIAL' => $params['CDFILIAL'],
                'P_NRORG' => $this->environment->getNrorg()
            ];
            
            return $this->supUtilities->crud($query, $params);
        } catch (Exception $e) {
            throw new \Exception('Erro ao buscar código da empresa!');
        }
    }
    
    public function getEmpresaInfo($params){    
        $query = "SELECT EMPRESA.CDEMPRESA, MIN(INSCRMUNICIP.CDINSCMUNI) AS CDINSCMUNI, MIN(INSCRESTAD.CDINSCESTA) AS CDINSCESTA FROM EMPRESA, INSCRMUNICIP, INSCRESTAD WHERE EMPRESA.CDEMPRESA = :P_CDEMPRESA AND EMPRESA.CDEMPRESA = INSCRMUNICIP.CDEMPRESA AND EMPRESA.CDEMPRESA = INSCRESTAD.CDEMPRESA AND EMPRESA.NRORG = INSCRMUNICIP.NRORG AND INSCRMUNICIP.NRORG = INSCRESTAD.NRORG AND INSCRESTAD.NRORG = :P_NRORG GROUP BY EMPRESA.CDEMPRESA, EMPRESA.NRINSCESTA";
        $params = [
            'P_CDEMPRESA'   => $params['CDEMPRESA'],
            'P_NRORG'       => $this->environment->getNrorg()
        ];
        return $this->supUtilities->crud($query, $params);
    }
    
    public function getContPzoCot($params){    
        $query = "SELECT COUNT(*) AS CTPZOS FROM PZOPGTITMCOL WHERE PZOPGTITMCOL.CDFILIAL =:P_CDFILIAL AND PZOPGTITMCOL.NRCOLEPRECO =:P_NRCOLEPRECO AND PZOPGTITMCOL.CDFORNECED =:P_CDFORNECED AND PZOPGTITMCOL.CDPRODUTO =:P_CDPRODUTO AND PZOPGTITMCOL.CDMARCA =:P_CDMARCA AND PZOPGTITMCOL.CDREGIAO =:P_CDREGIAO";
        $params = [
            "P_CDFILIAL"    => $params['CDFILIAL'],
            "P_NRCOLEPRECO" => $params['NRCOLEPRECO'],    
            "P_CDPRODUTO"   => $params['CDPRODUTO'],    
            "P_CDMARCA"     => $params['CDMARCA'],    
            "P_CDFORNECED"  => $params['CDFORNECED'],    
            "P_CDREGIAO"    => $params['CDREGIAO']
        ];
        return $this->supUtilities->crud($query, $params);
    }
    
    public function getPgtoProdForn($params){    
        $query = "SELECT PRODFORN.CDPRODUTO, PRODFORN.CDFORMPGTO, PRODFORN.IDTPFORPGPRF, PRODFORN.NRPARCPGTPRF, PRODFORN.NRDIAPGFIPRF, PRODFORN.IDUTDIAPGPRF FROM PRODFORN, PRODUTO WHERE PRODFORN.CDFORNECED =:P_CDFORNECED AND PRODFORN.CDPRODUTO = PRODUTO.CDPRODUTO AND PRODFORN.NRORG = PRODUTO.NRORG AND PRODUTO.NRORG = :P_NRORG AND PRODUTO.CDARVPROD = ( SELECT MAX(P.CDARVPROD) FROM PRODUTO P, PRODFORN WHERE PRODFORN.CDFORNECED =:P_CDFORNECED AND PRODFORN.CDPRODUTO = P.CDPRODUTO AND :P_CDPRODUTO LIKE PRODUTO.CDARVPROD || '%' AND P.NRORG = PRODFORN.NRORG AND PRODFORN.NRORG = :P_NRORG )";
        $params = [
            "P_CDPRODUTO"   => $params['CDPRODUTO'],  
            "P_CDFORNECED"  => $params['CDFORNECED'],
            'P_NRORG'       => $this->environment->getNrorg()
        ];
        return $this->supUtilities->crud($query, $params);
    }
    
    public function getPgtoForn($params){    
        $query = "SELECT CDFORMPGTO, IDTPFORPGTFO, NRPARCPGTOFO, NRDIAPGFIXFO, IDUTDIAPGTFO, IDTPDESCFORN, VRPEDESCFORN, IDTPFRETFORN FROM FORNECEDOR WHERE CDFORNECED =:P_CDFORNECED AND NRORG = :P_NRORG";
        $params = [
            "P_CDFORNECED"  => $params['CDFORNECED'],
            'P_NRORG'       => $this->environment->getNrorg()
        ];
        return $this->supUtilities->crud($query, $params);
    }
    
    public function getProdForn($params){    
        $query = "SELECT CDPRODUTO, CDARVPROD, NMPRODUTO, CDMARCA, NMMARCA, NRORG FROM ( SELECT DISTINCT PRODUTO.CDPRODUTO, PRODUTO.CDARVPROD, UPPER(RTRIM(PRODUTO.NMPRODUTO) ) AS NMPRODUTO, MARCA.CDMARCA, RTRIM(MARCA.NMMARCA) AS NMMARCA, PARASUPRI.NRORG FROM ITEMCOLE, PRMARCPEDCOL, ( SELECT 'N' AS ALTERADO FROM DUAL ) SUBTABELAALT, PRODUTO, MARCA, PARASUPRI WHERE ITEMCOLE.CDFILIAL =:P_CDFILIAL AND ITEMCOLE.NRCOLEPRECO =:P_NRCOLEPRECO AND ITEMCOLE.CDPRODUTO = PRODUTO.CDPRODUTO AND ITEMCOLE.CDFILIAL = PRMARCPEDCOL.CDFILIAL (+) AND ITEMCOLE.NRPEDIDOCOL = PRMARCPEDCOL.NRPEDIDOCOL (+) AND ITEMCOLE.CDPRODUTO = PRMARCPEDCOL.CDPRODUTO (+) AND ITEMCOLE.CDMARCA = PRMARCPEDCOL.CDMARCA (+) AND ITEMCOLE.CDMARCA = MARCA.CDMARCA AND ITEMCOLE.CDFILIAL = PARASUPRI.CDFILIAL AND ITEMCOLE.NRORG = PARASUPRI.NRORG AND PARASUPRI.NRORG =:P_NRORG ) ORDER BY DECODE(CDARVPROD,2,NMPRODUTO,NMPRODUTO), NMMARCA";
        $params = [
            "P_CDFILIAL"    => $params['CDFILIAL'],
            "P_NRCOLEPRECO" => $params['NRCOLEPRECO'],
            'P_NRORG'       => $this->environment->getNrorg()
        ];
        return $this->supUtilities->crud($query, $params);
    }
    
    public function validaFornCot($params){    
        $query = "SELECT NRINSJURFORN, CDFORNECED, IDTPIJURFORN, NMRAZSOCFORN, CAST(RTRIM(NMFANTFORN) AS VARCHAR(40) ) AS NMFANTFORN FROM FORNECEDOR WHERE INSTR(:P_CDFORNECED,FORNECEDOR.CDFORNECED) > 0 AND ( FORNECEDOR.NRINSJURFORN =:P_NRINSJURFORN OR :P_NRINSJURFORN = 'T' ) AND FORNECEDOR.NRORG =:P_NRORG ORDER BY FORNECEDOR.IDTPIJURFORN, FORNECEDOR.NRINSJURFORN";
        $params = [
            "P_CDFORNECED"  => $params['CDFORNECED'],
            "P_NRINSJURFORN"=> $params['NRINSJURFORN'],
            'P_NRORG'       => $this->environment->getNrorg()
        ];
        return $this->supUtilities->crud($query, $params);
    }
    
    public function validaProdForn($paramsRow, $paramsProd){    
        $query = "SELECT CDPRODUTO FROM PRODFORNCOMP, PARASUPRI WHERE PARASUPRI.CDFILIAL = :P_CDFILIAL AND PRODFORNCOMP.CDFORNECED = :P_CDFORNECED AND PRODFORNCOMP.CDPRODCOMPL = :P_CDPRODUTO AND PRODFORNCOMP.NRORG = PARASUPRI.NRORG AND PARASUPRI.NRORG = :P_NRORG AND NOT EXISTS (SELECT QFOPRODFORN.CDPRODUTO FROM QFOPRODFORN WHERE (QFOPRODFORN.CDPRODUTO = :P_CDPRODUTO) AND (QFOPRODFORN.CDFORNECED= :P_CDFORNECED) AND (QFOPRODFORN.NRORG = :P_NRORG) AND ((QFOPRODFORN.IDFORNAPROV = 'N' AND QFOPRODFORN.IDFORNAUTOR = 'N') OR ((QFOPRODFORN.IDFORNAPROV = 'S') AND ((PARASUPRI.IDULTIDTVALQ = 'M' AND :P_DTCOL > LAST_DAY(QFOPRODFORN.DTVALPRFORN)) OR (PARASUPRI.IDULTIDTVALQ = 'D' AND :P_DTCOL > QFOPRODFORN.DTVALPRFORN) ) ) ) )";
        $params = [
            "P_CDFILIAL"    => $paramsRow['CDFILIAL'],
            "P_CDFORNECED"  => $paramsRow['CDFORNECED'],
            "P_CDPRODUTO"   => $paramsProd['CDPRODUTO'],
            "P_DTCOL"       => $paramsRow['DTCOLEPREC'],
            'P_NRORG'       => $this->environment->getNrorg()
        ];
        return $this->supUtilities->crud($query, $params);
    }
    
    public function verificaLista($param){    
        $query = "SELECT COUNT(*) AS CONTA FROM ITEMCOLE WHERE CDFILIAL =:P_CDFILIAL AND NRCOLEPRECO = :P_NRCOLEPRECO AND CDREGIAO = NVL(:P_CDREGIAO, CDREGIAO) AND CDFORNECED = NVL(:P_CDFORNECED, CDFORNECED) AND CDPRODUTO = NVL(:P_CDPRODUTO, CDPRODUTO) AND CDMARCA = NVL(:P_CDMARCA,CDMARCA) AND IDITMLISPREC = 'S' AND NRORG = :P_NRORG";
        $params = [
            "P_CDFILIAL"    => $param['CDFILIAL'],
            "P_NRCOLEPRECO" => $param['NRCOLEPRECO'],
            "P_CDREGIAO"    => $param['CDREGIAO'],
            "P_CDFORNECED"  => $param['CDFORNECED'],
            "P_CDPRODUTO"   => $param['CDPRODUTO'],
            "P_CDMARCA"     => $param['CDMARCA'],
            'P_NRORG'       => $this->environment->getNrorg()
        ];
        return $this->supUtilities->crud($query, $params);
    }
    
    public function getProdInfoToDelete($params){    
        $query = "SELECT ITEMCOLE.CDFILIAL, ITEMCOLE.NRCOLEPRECO, ITEMCOLE.CDMARCA, ITEMCOLE.CDREGIAO, ITEMCOLE.CDFORNECED, ITEMCOLE.CDPRODUTO, ITEMCOLE.NRSEQPRELIST FROM ITEMCOLE WHERE CDFILIAL = :P_CDFILIAL AND NRCOLEPRECO = :P_NRCOLEPRECO AND (CDPRODUTO = :P_CDPRODUTO OR :P_CDPRODUTO = 'T') AND (CDREGIAO = :P_CDREGIAO OR :P_CDREGIAO = 'T') AND (CDFORNECED = :P_CDFORNECED OR :P_CDFORNECED = 'T') AND ( CDMARCA =:P_CDMARCA OR :P_CDMARCA = 'T' ) AND (NRORG = :P_NRORG)";
        $params = [
            "P_CDFILIAL"    => $params['CDFILIAL'],
            "P_NRCOLEPRECO" => $params['NRCOLEPRECO'],
            "P_CDPRODUTO"   => $params['CDPRODUTO'],
            "P_CDREGIAO"    => '001',
            "P_CDFORNECED"  => $params['CDFORNECED'],
            "P_CDMARCA"     => '00206',
            'P_NRORG'       => $this->environment->getNrorg()
        ];
        return $this->supUtilities->crud($query, $params);
    }
    
    public function getQualifMarca($params){    
        $query = "SELECT QFOMARPROD.CDMARCA FROM QFOMARPROD, PARASUPRI WHERE (QFOMARPROD.CDPRODUTO = :P_CDPRODUTO) AND (QFOMARPROD.CDMARCA = :P_CDMARCA) AND (PARASUPRI.CDFILIAL = :P_CDFILIAL) AND (QFOMARPROD.NRORG = PARASUPRI.NRORG) AND (PARASUPRI.NRORG = :P_NRORG) AND ((QFOMARPROD.IDAPRMARPRO = 'N' AND QFOMARPROD.IDAUTMARPRO = 'N') OR ((QFOMARPROD.IDAPRMARPRO = 'S') AND ((PARASUPRI.IDULTIDTVALQ = 'M' AND :P_DTCOL > LAST_DAY(QFOMARPROD.DTVALAPROVA)) OR (PARASUPRI.IDULTIDTVALQ = 'D' AND :P_DTCOL > QFOMARPROD.DTVALAPROVA)) ) ) UNION SELECT PARASUPRI.CDMARCA FROM PARASUPRI WHERE PARASUPRI.CDFILIAL = :P_CDFILIAL AND PARASUPRI.IDMARSEMQUA = 'S' AND PARASUPRI.NRORG = :P_NRORG AND NOT EXISTS (SELECT QFOMARPROD.CDMARCA FROM QFOMARPROD WHERE QFOMARPROD.CDPRODUTO = :P_CDPRODUTO AND QFOMARPROD.CDMARCA = :P_CDMARCA AND QFOMARPROD.NRORG = :P_NRORG)";
        $params = [
            "P_CDFILIAL"    => $params['CDFILIAL'],
            "P_CDPRODUTO"   => $params['CDPRODUTO'],
            "P_CDMARCA"     => $params['CDMARCA'],
            "P_DTCOL"       => $params['DTCOLEPREC'],
            'P_NRORG'       => $this->environment->getNrorg()
        ];
        return $this->supUtilities->crud($query, $params);
    }
    
    public function getVerQfoProdForn($params){    
        $query = "SELECT FORNECEDOR.CDFORNECED FROM FORNECEDOR, PRODUTO WHERE (PRODUTO.CDPRODUTO = :P_CDPRODINI) AND (PRODUTO.CDPRODESTO IS NOT NULL) AND (FORNECEDOR.CDFORNECED = :P_CDFORNECED) AND (FORNECEDOR.NRORG = PRODUTO.NRORG) AND (PRODUTO.NRORG = :P_NRORG) AND NOT EXISTS (SELECT QFOPRODFORN.CDPRODUTO FROM QFOPRODFORN, PARASUPRI WHERE PARASUPRI.CDFILIAL = :P_CDFILIAL AND (QFOPRODFORN.CDPRODUTO = PRODUTO.CDPRODUTO) AND (QFOPRODFORN.CDFORNECED= FORNECEDOR.CDFORNECED) AND (QFOPRODFORN.NRORG = PARASUPRI.NRORG) AND (PARASUPRI.NRORG = :P_NRORG) AND ((QFOPRODFORN.IDFORNAPROV = 'N' AND QFOPRODFORN.IDFORNAUTOR = 'N') OR ((QFOPRODFORN.IDFORNAPROV = 'S') AND ((PARASUPRI.IDULTIDTVALQ = 'M' AND :P_DTCOL > LAST_DAY(QFOPRODFORN.DTVALPRFORN)) OR (PARASUPRI.IDULTIDTVALQ = 'D' AND :P_DTCOL > QFOPRODFORN.DTVALPRFORN)) ) ))";
        $params = [
            "P_CDFILIAL"    => $params['CDFILIAL'],
            "P_CDPRODINI"   => $params['CDPRODUTO'],
            "P_CDFORNECED"  => $params['CDFORNECED'],
            "P_DTCOL"       => $params['DTCOLEPREC'],
            'P_NRORG'       => $this->environment->getNrorg()
        ];
        return $this->supUtilities->crud($query, $params);
    }
    
    public function validaFuncFili($params){    
        $query = "SELECT IDFUNCAOFILI FROM FILIAL WHERE CDFILIAL = :P_CDFILIPROC AND NRORG = :P_NRORG";
        $params = [
            "P_CDFILIPROC"  => $params['CDFILIPROC'],
            'P_NRORG'       => $this->environment->getNrorg()
        ];
        return $this->supUtilities->crud($query, $params);
    }
    
    public function getFiliReg($params){    
        $query = "SELECT CDFILICTRL FROM PARAMPRODSUP WHERE CDFILICTRL = :P_CDFILICTRL AND IDFILINEGREG = 'S' AND CDREGIAO = :P_CDREGIAO AND CDPRODUTO = :P_CDPRODUTO AND NRORG = :P_NRORG";
        $params = [
            "P_CDFILIPROC"  => $params['CDFILIPROC'],
            'P_NRORG'       => $this->environment->getNrorg()
        ];
        return $this->supUtilities->crud($query, $params);
    }
    
    public function getFiliCor($params){    
        $query = "SELECT CDFILICTRL FROM PARAMPRODSUP WHERE CDFILICTRL =:P_CDFILICTRL AND IDFILINEGCOR = 'S' AND CDPRODUTO =:P_CDPRODUTO AND CDREGIAO =:P_CDREG AND NRORG = :P_NRORG";
        $params = [
            "P_CDFILICTRL"  => $params['CDFILIPROC'],
            "P_CDPRODUTO"   => $params['CDPRODUTO'],
            "P_CDREG"       => $params['CDREGIAO'],
            'P_NRORG'       => $this->environment->getNrorg()
        ];
        return $this->supUtilities->crud($query, $params);
    }
    
    public function validaProdutoFornecedor($params){ 
        $query = "SELECT CDPRODUTO FROM PRODFORNCOMP WHERE PRODFORNCOMP.CDFORNECED = :P_CDFORNECED AND PRODFORNCOMP.CDPRODCOMPL = :P_CDPRODUTO AND PRODFORNCOMP.NRORG = :P_NRORG";
        $params = [
            "P_CDFORNECED"  => $params['CDFORNECED'],
            "P_CDPRODUTO"   => $params['CDPRODUTO'],
            'P_NRORG'       => $this->environment->getNrorg()
        ];
        return $this->supUtilities->crud($query, $params);
    }
    
    public function validaMarcaProduto($params){    
        $query = "SELECT CDPRODUTO FROM MARCPRODCOMP WHERE CDPRMARCOMPL = :P_CDPRODUTO AND CDMARCA = :P_CDMARCA AND NRORG = :P_NRORG";
        $params = [
            "P_CDPRODUTO"   => $params['CDPRODUTO'],
            "P_CDMARCA"     => $params['CDMARCA'],
            'P_NRORG'       => $this->environment->getNrorg()
        ];
        return $this->supUtilities->crud($query, $params);
    }
    
    public function getCodEAN($params){    
        $query = "SELECT NVL(IDOBRIGAEAN, 'N') AS IDOBRIGAEAN FROM FORNECEDOR WHERE CDFORNECED = :P_CDFORNECED AND NRORG = :P_NRORG";
        $params = [
            "P_CDFORNECED"  => $params['CDFORNECED'],
            'P_NRORG'       => $this->environment->getNrorg()
        ];
        return $this->supUtilities->crud($query, $params);
    }
    
    public function verificaExisteEAN($params){    
        $query = "SELECT CDPRODCOMP, CDEAN, CDMARCA FROM MARCPRODUTO WHERE CDPRODCOMP =:P_CDPRODUTO AND CDMARCA =:P_CDMARCA AND NRORG = :P_NRORG";
        $params = [
            "P_CDPRODUTO"   => $params['CDPRODUTO'],
            "P_CDMARCA"     => $params['CDMARCA'],
            'P_NRORG'       => $this->environment->getNrorg()
        ];
        return $this->supUtilities->crud($query, $params);
    }
    
    public function getConsCotAn($params){    
        $query = "SELECT C.NRCOLEPRECO G FROM COLETAPR C INNER JOIN ITEMCOLE IC ON IC.CDFILIAL = C.CDFILIAL AND IC.NRCOLEPRECO = C.NRCOLEPRECO INNER JOIN REGIAO R ON IC.CDREGIAO = R.CDREGIAO WHERE ( ( INSTR(:P_CDOPERADOR,C.CDOPERADOR) <> 0 ) OR ( (:P_TCDOPERADOR = 'T' ) AND C.CDOPERADOR IN ( ( SELECT CM.CDOPERADOR FROM COMPROPER CO INNER JOIN COMPRADOR CM ON CO.CDCOMPRADOR = CM.CDOPERADOR INNER JOIN OPERADOR O ON O.CDOPERADOR = CM.CDOPERADOR WHERE CO.IDCOTCMPOPER = 'S' AND CO.CDOPERADOR =:P_CDOPERADOR UNION SELECT CM.CDOPERADOR FROM COMPRADOR CM INNER JOIN OPERADOR O ON CM.CDOPERADOR = O.CDOPERADOR WHERE CM.CDOPERADOR =:P_CDOPERADOR ) ) ) ) AND ( C.CDFILIAL =:P_CDFILIAL ) AND (C.NRORG = IC.NRORG) AND (IC.NRORG = :P_NRORG) AND ( ( :P_IDDUPLICA = '1' AND C.DTINIVALCOL >=:P_DTINIVALCOL AND C.DTFINVALCOL <=:P_DTFINVALCOL ) OR ( :P_IDDUPLICA = '2' AND C.NRCOLEPRECO =:P_NRCOLEDUP ) ) AND INSTR(NVL(:P_CDREGIAO,R.CDREGIAO),R.CDREGIAO) <> 0 AND EXISTS ( SELECT 1 V FROM ITEMCOLE I INNER JOIN COLETAPR CC ON CC.CDFILIAL = I.CDFILIAL AND CC.NRCOLEPRECO = I.NRCOLEPRECO WHERE I.CDFILIAL = IC.CDFILIAL AND I.NRCOLEPRECO <> IC.NRCOLEPRECO AND I.CDMARCA = IC.CDMARCA AND I.CDREGIAO = IC.CDREGIAO AND I.CDFORNECED = IC.CDFORNECED AND I.CDPRODUTO = IC.CDPRODUTO AND I.NRORG = CC.NRORG AND CC.NRORG = :P_NRORG AND CC.DTINIVALCOL =:P_DTININOVA AND CC.DTFINVALCOL =:P_DTFINNOVA AND NVL(CC.IDSITUCOLETA,'C') = 'A' ) AND ROWNUM = 1";
        $params = [
            "P_CDFILIAL"      => $params['CDFILIAL'],
            "P_DTINIVALCOL"   => $params['DTINIVALCOL'],
            "P_DTFINVALCOL"   => $params['DTFINVALCOL'],
            "P_DTININOVA"     => $params['DTNOVINIVAL'],
            "P_DTFINNOVA"     => $params['DTNOVFINVAL'],
            "P_CDOPERADOR"    => $params['CDOPERADOR'],
            "P_TCDOPERADOR"   => $params['TCDOPERADOR'],
            "P_NRCOLEDUP"     => $params['NRCOLEDUP'],
            "P_CDREGIAO"      => $params['CDREGIAO'],
            "P_IDDUPLICA"     => $params['IDDUPLICA'],
            'P_NRORG'         => $this->environment->getNrorg()
        ];
        return $this->supUtilities->crud($query, $params);
    }
    
    public function getCotacoesValidade($params){    
        $query = "SELECT COLETAPR.CDFILIAL, COLETAPR.NRCOLEPRECO FROM COLETAPR, PARASUPRI WHERE COLETAPR.DTINIVALCOL = :P_DTINIANTERIOR AND COLETAPR.DTFINVALCOL = :P_DTFINANTERIOR AND ((INSTR(:P_CDCOMPRADOR, COLETAPR.CDOPERADOR) > 0) OR (:P_CDCOMPRADOR = 'T')) AND COLETAPR.CDFILIAL = :P_CDFILCENTCMP AND ((INSTR(:P_COTACAO, COLETAPR.NRCOLEPRECO) > 0) OR (:P_COTACAO = 'T')) AND PARASUPRI.CDFILIAL = :P_CDFILCENTCMP AND ((COLETAPR.IDSITUCOLETA = 'A') OR (NVL(PARASUPRI.IDUTISTATCOT, 'N') = 'N')) AND COLETAPR.NRORG = PARASUPRI.NRORG AND PARASUPRI.NRORG = :P_NRORG AND EXISTS (SELECT 1 FROM ITEMCOLE WHERE ITEMCOLE.CDFILIAL = :P_CDFILCENTCMP AND ITEMCOLE.NRCOLEPRECO = COLETAPR.NRCOLEPRECO AND ((INSTR(:P_CDFORNECED, ITEMCOLE.CDFORNECED) > 0) OR (:P_CDFORNECED = 'T')) AND ITEMCOLE.DTINIVALITM = :P_DTINIANTERIOR AND ITEMCOLE.DTFINVALITM = :P_DTFINANTERIOR AND ITEMCOLE.NRORG = :P_NRORG )";
        $params = [
            "P_DTINIANTERIOR" => $params['DTINIVALCOL'],
            "P_DTFINANTERIOR" => $params['DTFINVALCOL'],
            "P_CDCOMPRADOR"   => $params['CDCOMPRADOR'],
            "P_CDFILCENTCMP"  => $params['CDFILCENTCMP'],
            "P_COTACAO"       => $params['COTACAO'],
            "P_CDFORNECED"    => $params['CDFORNECED'],
            'P_NRORG'         => $this->environment->getNrorg()
        ];
        return $this->supUtilities->crud($query, $params);
    }
    
    public function getImpostoCotacao($params)
    {
        try {
            $query = "SELECT ITEMCOLE.CDFILIAL, ITEMCOLE.NRCOLEPRECO, ITEMCOLE.CDPRODUTO, PRODUTO.CDARVPROD, ITEMCOLE.CDMARCA, ITEMCOLE.CDFORNECED, ITEMCOLE.CDREGIAO, ITEMCOLE.CDTIPCOB, TIPOCOBR.NMTIPOCOBR, ITEMCOLE.VRPRECUNIITM, ITEMCOLE.VRPRECECOITM, ITEMCOLE.VRPERDESITM, ITEMCOLE.QTACMPITMCOL, ITEMCOLE.CDFORMPGTO, FORMPGTO.NMFORMPGTO, ITEMCOLE.IDTPFOPGTCOL, ITEMCOLE.NRPARCPGTCOL, ITEMCOLE.NRDIAPGFXCOL, ITEMCOLE.IDUTDIAPGCOL, ITEMCOLE.IDTIPFREITM, ITEMCOLE.NRPZOENTCOL, ITEMCOLE.DTINIVALITM, ITEMCOLE.DTFINVALITM, ITEMCOLE.CDSEGMARCCOL, MARCA.NMMARCA, NVL(ITEMCOLE.VRFRETEFORNEPLANCOT,FORNECEDOR.VRFRETEFORN) AS VRFRETEFORN FROM ITEMCOLE, FORNECEDOR, PRODUTO, TIPOCOBR, FORMPGTO, MARCA WHERE FORNECEDOR.CDFORNECED = ITEMCOLE.CDFORNECED AND PRODUTO.CDPRODUTO = ITEMCOLE.CDPRODUTO AND ITEMCOLE.CDREGIAO =:P_CDREGIAO AND ITEMCOLE.CDFILIAL =:P_CDFILIAL AND ITEMCOLE.NRCOLEPRECO =:P_NRCOLEPRECO AND ITEMCOLE.CDPRODUTO BETWEEN :P_CDPRODINI AND :P_CDPRODFIN AND ITEMCOLE.CDFORNECED =:P_CDFORNECED AND ITEMCOLE.CDSEGMARCCOL = MARCA.CDMARCA (+) AND ITEMCOLE.CDTIPCOB = TIPOCOBR.CDTIPCOB (+) AND ITEMCOLE.CDFORMPGTO = FORMPGTO.CDFORMPGTO (+)";
            
            $params = [
                "P_CDFILIAL"      => $params['CDFILIAL'],
                "P_NRCOLEPRECO"   => $params['NRCOLEPRECO'],
                "P_CDFORNECED"    => $params['CDFORNECED'],
                "P_CDPRODINI"     => $params['CDPRODINI'],
                "P_CDPRODFIN"     => $params['CDPRODFIN'],
                "P_CDREGIAO"      => $params['CDREGIAO']
            ];
            
            return $this->supUtilities->crud($query, $params);
        } catch (Exception $e) {
            throw new \Exception('Erro ao buscar dados para calcular impostos!');
        }
    }
    
    public function getItemCole($params){    
        $query = "SELECT ITEMCOLE.CDFILIAL, ITEMCOLE.NRCOLEPRECO, ITEMCOLE.CDPRODUTO, PRODUTO.CDARVPROD, ITEMCOLE.CDMARCA, ITEMCOLE.CDFORNECED, ITEMCOLE.CDREGIAO, ITEMCOLE.CDTIPCOB, TIPOCOBR.NMTIPOCOBR, ITEMCOLE.VRPRECUNIITM, ITEMCOLE.VRPRECECOITM, ITEMCOLE.VRPERDESITM, ITEMCOLE.QTACMPITMCOL, ITEMCOLE.CDFORMPGTO, FORMPGTO.NMFORMPGTO, ITEMCOLE.IDTPFOPGTCOL, ITEMCOLE.NRPARCPGTCOL, ITEMCOLE.NRDIAPGFXCOL, ITEMCOLE.IDUTDIAPGCOL, ITEMCOLE.IDTIPFREITM, ITEMCOLE.NRPZOENTCOL, ITEMCOLE.DTINIVALITM, ITEMCOLE.DTFINVALITM, ITEMCOLE.CDSEGMARCCOL, MARCA.NMMARCA, NVL(ITEMCOLE.VRFRETEFORNEPLANCOT,FORNECEDOR.VRFRETEFORN) AS VRFRETEFORN FROM ITEMCOLE, FORNECEDOR, PRODUTO, TIPOCOBR, FORMPGTO, MARCA WHERE FORNECEDOR.CDFORNECED = ITEMCOLE.CDFORNECED AND PRODUTO.CDPRODUTO = ITEMCOLE.CDPRODUTO AND ITEMCOLE.CDREGIAO =:P_CDREGIAO AND ITEMCOLE.CDFILIAL =:P_CDFILIAL AND ITEMCOLE.NRCOLEPRECO =:P_NRCOLEPRECO AND ITEMCOLE.CDPRODUTO =:P_CDPRODUTO AND ITEMCOLE.CDMARCA =:P_CDMARCA AND ITEMCOLE.CDFORNECED =:P_CDFORNECED AND ITEMCOLE.CDSEGMARCCOL = MARCA.CDMARCA (+) AND ITEMCOLE.CDTIPCOB = TIPOCOBR.CDTIPCOB (+) AND ITEMCOLE.CDFORMPGTO = FORMPGTO.CDFORMPGTO (+)";
        $params = [
            "P_CDFILIAL"      => $params['CDFILIAL'],
            "P_NRCOLEPRECO"   => $params['NRCOLEPRECO'],
            "P_CDFORNECED"    => $params['CDFORNECED'],
            "P_CDPRODUTO"     => $params['CDPRODUTO'],
            "P_CDMARCA"       => $params['CDMARCA'],
            "P_CDREGIAO"      => $params['CDREGIAO']
        ];
        return $this->supUtilities->crud($query, $params);
    }
    
    public function verificaColetaApr($params){    
        $query = "SELECT COLETAPR.CDFILIAL, COLETAPR.NRCOLEPRECO FROM COLETAPR WHERE (COLETAPR.CDFILIAL = :P_CDFILIAL) AND (COLETAPR.NRCOLEPRECO = :P_NRCOLEPRECO) AND (COLETAPR.NRORG = :P_NRORG)";
        $params = [
            "P_CDFILIAL"      => $params['CDFILIAL'],
            "P_NRCOLEPRECO"   => $params['NRCOLEPRECO'],
            'P_NRORG'         => $this->environment->getNrorg()
        ];
        return $this->supUtilities->crud($query, $params);
    }
    
    public function pesqMarca($params){    
        $query = "SELECT DISTINCT MARCPROD.CDMARCA, MARCA.NMMARCA FROM MARCPROD, PRODUTO, MARCA WHERE :P_CDARVPROD LIKE PRODUTO.CDARVPROD || '%' AND ((INSTR(:P_CDMARCA, MARCPROD.CDMARCA)<>0) OR (:P_CDMARCA = 'T')) AND MARCPROD.CDMARCA = MARCA.CDMARCA AND MARCPROD.CDPRODUTO = PRODUTO.CDPRODUTO AND MARCPROD.NRORG = PRODUTO.NRORG AND PRODUTO.NRORG = MARCA.NRORG AND MARCA.NRORG = :P_NRORG ORDER BY MARCPROD.CDMARCA";
        $params = [
            "P_CDMARCA"       => $params['CDMARCA'],
            "P_CDARVPROD"     => $params['CDARVPROD'],
            'P_NRORG'         => $this->environment->getNrorg()
        ];
        return $this->supUtilities->crud($query, $params);
    }
    
    public function getMarcaAutorizada($params){    
        $query = "SELECT QFOMARPROD.CDMARCA FROM QFOMARPROD, PARASUPRI WHERE (QFOMARPROD.CDPRODUTO = :P_CDPRODUTO) AND (QFOMARPROD.CDMARCA = :P_CDMARCA) AND (PARASUPRI.CDFILIAL = :P_CDCENTCMP) AND (QFOMARPROD.NRORG = PARASUPRI.NRORG) AND (PARASUPRI.NRORG = :P_NRORG) AND ((QFOMARPROD.IDAPRMARPRO = 'N' AND QFOMARPROD.IDAUTMARPRO = 'N') OR ((QFOMARPROD.IDAPRMARPRO = 'S' OR (QFOMARPROD.IDAPRMARPRO = 'N' AND QFOMARPROD.IDAUTMARPRO = 'S')) AND ((PARASUPRI.IDULTIDTVALQ = 'M' AND TO_DATE(:P_DTCOLETA) > LAST_DAY(QFOMARPROD.DTVALAPROVA)) OR (PARASUPRI.IDULTIDTVALQ = 'D' AND TO_DATE(:P_DTCOLETA) > QFOMARPROD.DTVALAPROVA)) )) UNION SELECT PARASUPRI.CDMARCA FROM PARASUPRI WHERE PARASUPRI.CDFILIAL = :P_CDCENTCMP AND PARASUPRI.IDMARSEMQUA = 'S' AND PARASUPRI.NRORG = :P_NRORG AND NOT EXISTS (SELECT QFOMARPROD.CDMARCA FROM QFOMARPROD WHERE QFOMARPROD.CDPRODUTO = :P_CDPRODUTO AND QFOMARPROD.CDMARCA = :P_CDMARCA AND QFOMARPROD.NRORG = :P_NRORG)";
        $params = [
            "P_CDPRODUTO"     => $params['CDPRODUTO'],
            "P_CDMARCA"       => $params['CDMARCASEC'],
            "P_CDCENTCMP"     => $params['CDFILCENT'],
            "P_DTCOLETA"      => $params['DTCOLEPREC'],
            'P_NRORG'         => $this->environment->getNrorg()
        ];
        return $this->supUtilities->crud($query, $params);
    }
    
    public function getTipoCobranca($params){    
        $query = "SELECT CDTIPCOB, NMTIPOCOBR, NMREDTIPCOB FROM TIPOCOBR WHERE (CDTIPCOB = :P_CDTIPCOB OR :P_TCDTIPCOB = 'T') AND (NRORG = :P_NRORG)";
        $params = [
            "P_CDTIPCOB"      => $params['CDTIPCOB'],
            "P_TCDTIPCOB"     => $params['TCDTIPCOB'],
            'P_NRORG'         => $this->environment->getNrorg()
        ];
        return $this->supUtilities->crud($query, $params);
    }
    
    public function getSomaPercent($params){    
        $query = "SELECT SUM(PZOPGTITMCOL.VRPERCPGTCOL) AS SOMAPERC FROM PZOPGTITMCOL WHERE PZOPGTITMCOL.CDFILIAL = :P_CDFILIAL AND PZOPGTITMCOL.NRCOLEPRECO = :P_NRCOLEPRECO AND PZOPGTITMCOL.CDFORNECED = :P_CDFORNECED AND PZOPGTITMCOL.CDPRODUTO = :P_CDPRODUTO AND PZOPGTITMCOL.CDMARCA = :P_CDMARCA AND PZOPGTITMCOL.CDREGIAO = :P_CDREGIAO AND PZOPGTITMCOL.NRORG = :P_NRORG";
        $params = [
            "P_CDFILIAL"      => $params['CDFILIAL'],
            "P_NRCOLEPRECO"   => $params['NRCOLEPRECO'],
            "P_CDFORNECED"    => $params['CDFORNECED'],
            "P_CDPRODUTO"     => $params['CDPRODUTO'],
            "P_CDMARCA"       => $params['CDMARCA'],
            "P_CDREGIAO"      => $params['CDREGIAO'],
            'P_NRORG'         => $this->environment->getNrorg()
        ];
        return $this->supUtilities->crud($query, $params);
    }
    
    public function verificaPrecoZerado($params){    
        $query = "SELECT TOTITM.CTTOTITM, TOTZERO.CTTOTZERO FROM (SELECT COUNT(*) AS CTTOTITM FROM ITEMCOLE WHERE CDFILIAL = :P_CDFILIAL AND NRORG = :P_NRORG AND NRCOLEPRECO = :P_NRCOLEPRECO) TOTITM, (SELECT COUNT(*) AS CTTOTZERO FROM ITEMCOLE WHERE CDFILIAL = :P_CDFILIAL AND NRCOLEPRECO = :P_NRCOLEPRECO AND NRORG = :P_NRORG AND NVL(VRPRECUNIITM,0) = 0) TOTZERO";
        $params = [
            "P_CDFILIAL"      => $params['CDFILIAL'],
            "P_NRCOLEPRECO"   => $params['NRCOLEPRECO'],
            'P_NRORG'         => $this->environment->getNrorg()
        ];
        return $this->supUtilities->crud($query, $params);
    }
    
    public function verificaItemCole($params){    
        $query = "SELECT IT.CDFILIAL, IT.NRCOLEPRECO, IT.CDPRODUTO, IT.CDMARCA, IT.CDFORNECED, IT.CDREGIAO, PR.CDARVPROD FROM ITEMCOLE IT, PRODUTO PR WHERE IT.CDPRODUTO = PR.CDPRODUTO AND IT.VRPRECUNIITM > 0 AND IT.CDFILIAL = :P_CDFILIAL AND IT.NRCOLEPRECO = :P_NRCOLEPRECO AND IT.NRORG = PR.NRORG AND PR.NRORG = :P_NRORG";
        $params = [
            "P_CDFILIAL"      => $params['CDFILIAL'],
            "P_NRCOLEPRECO"   => $params['NRCOLEPRECO'],
            'P_NRORG'         => $this->environment->getNrorg()
        ];
        return $this->supUtilities->crud($query, $params);
    }
    
    public function getItemColeNegociacao($params){    
        $query = "SELECT ITEMCOLE.CDFILIAL, ITEMCOLE.NRCOLEPRECO, ITEMCOLE.CDFORNECED, ITEMCOLE.CDPRODUTO, ITEMCOLE.CDMARCA, ITEMCOLE.CDREGIAO, ITEMCOLE.IDTPFOPGTCOL, ITEMCOLE.NRDIAPGFXCOL, ITEMCOLE.NRPARCPGTCOL, ITEMCOLE.IDUTDIAPGCOL, ITEMCOLE.CDFORMPGTO, FORMPGTO.NMFORMPGTO, ITEMCOLE.VRPRECUNIITM, ITEMCOLE.VRPRECECOITM, ITEMCOLE.VRPERDESITM, ITEMCOLE.DTINIVALITM, ITEMCOLE.DTFINVALITM, ITEMCOLE.IDITMCOTPREC, ITEMCOLE.CDTIPCOB, ITEMCOLE.IDTIPFREITM, ITEMCOLE.VRPRECUNORIG, ITEMCOLE.VRPRECECORIG, ITEMCOLE.VRPRECPIORIG, ITEMCOLE.VRPRECOITMPI FROM ITEMCOLE, FORMPGTO WHERE ITEMCOLE.CDREGIAO = :P_CDREGIAO AND ITEMCOLE.CDFILIAL = :P_CDFILIAL AND ITEMCOLE.NRCOLEPRECO = :P_NRCOLEPRECO AND ITEMCOLE.CDPRODUTO = :P_CDPRODUTO AND ITEMCOLE.CDMARCA = :P_CDMARCA AND ITEMCOLE.CDFORNECED = :P_CDFORNECED AND ITEMCOLE.CDFORMPGTO = FORMPGTO.CDFORMPGTO(+)";
        $params = [
            "P_CDFILIAL"      => $params['CDFILIAL'],
            "P_NRCOLEPRECO"   => $params['NRCOLEPRECO'],
            "P_CDFORNECED"    => $params['CDFORNECED'],
            "P_CDPRODUTO"     => $params['CDPRODUTO'],
            "P_CDMARCA"       => $params['CDMARCA'],
            "P_CDREGIAO"      => $params['CDREGIAO']
        ];
        return $this->supUtilities->crud($query, $params);
    }
    
    public function getUltimaCotacaoFornecedor($params){    
        $query = "SELECT ITEMCOLE.VRPRECUNIITM, ITEMCOLE.VRPRECECOITM, ITEMCOLE.IDTPFOPGTCOL, ITEMCOLE.NRDIAPGFXCOL, ITEMCOLE.NRPARCPGTCOL, ITEMCOLE.IDUTDIAPGCOL, ITEMCOLE.CDFORMPGTO, FORMPGTO.NMFORMPGTO, ITEMCOLE.NRCOLEPRECO, ITEMCOLE.CDREGIAO FROM ITEMCOLE, FORMPGTO WHERE ITEMCOLE.CDFILIAL = :P_CDFILIAL AND ITEMCOLE.CDPRODUTO = :P_CDPRODUTO AND ITEMCOLE.CDFORNECED = :P_CDFORNECED AND ITEMCOLE.CDMARCA = :P_CDMARCA AND ITEMCOLE.CDREGIAO = :P_CDREGIAO AND ITEMCOLE.NRORG = FORMPGTO.NRORG AND FORMPGTO.NRORG = :P_NRORG AND ITEMCOLE.CDFORMPGTO = FORMPGTO.CDFORMPGTO(+) AND ITEMCOLE.NRCOLEPRECO= (SELECT MAX(NRCOLEPRECO) AS COLETA FROM ITEMCOLE WHERE CDFILIAL = :P_CDFILIAL AND NRCOLEPRECO <> :P_NRCOLEPRECO AND CDPRODUTO = :P_CDPRODUTO AND CDMARCA = :P_CDMARCA AND CDREGIAO = :P_CDREGIAO AND CDFORNECED = :P_CDFORNECED )";
        $params = [
            "P_CDFILIAL"      => $params['CDFILIAL'],
            "P_NRCOLEPRECO"   => $params['NRCOLEPRECO'],
            "P_CDFORNECED"    => $params['CDFORNECED'],
            "P_CDPRODUTO"     => $params['CDPRODUTO'],
            "P_CDMARCA"       => $params['CDMARCA'],
            "P_CDREGIAO"      => $params['CDREGIAO'],
            'P_NRORG'         => $this->environment->getNrorg()
        ];
        return $this->supUtilities->crud($query, $params);
    }
    
    public function getUltimaCompra($params){    
        $query = "SELECT AUTOFORNE.DTAUTOFORN, AUTOFORNE.CDFORNECED, FORNECEDOR.NMFANTFORN, AUTOFORNE.CDFORMPGTO, FORMPGTO.NMFORMPGTO, AUTOFORNE.IDTPFORPGTAF, AUTOFORNE.NRDIAPGFIXAF, AUTOFORNE.NRPARCPGTOAF, AUTOFORNE.IDUTDIAPGTAF, ITEMAUTF.VRPREUNIAF, ITEMAUTF.VRPERDESAF, ITEMAUTF.CDFILDEST, ITEMAUTF.CDFILLANC, ITEMAUTF.NRAUTOFORN, ITEMAUTF.DTPROGENTAF FROM AUTOFORNE, ITEMAUTF, FORNECEDOR, FORMPGTO, ( SELECT MAX(NRAUTOFORN) AS NRAUTOFORN, CDFILDEST FROM ITEMAUTF, ( SELECT FILIAL.CDFILIAL FROM FILIAL WHERE FILIAL.CDFILCENTCMP = :P_CDCENTCMP UNION SELECT CDFILIAL FROM FILIAL WHERE CDFILIAL = :P_CDCENTCMP ) FIL_REG WHERE CDPRODUTO = :P_CDPRODUTO AND CDMARCA = :P_CDMARCA AND CDFILDEST = FIL_REG.CDFILIAL GROUP BY CDFILDEST ) ULT_AF WHERE ITEMAUTF.NRAUTOFORN = ULT_AF.NRAUTOFORN AND ITEMAUTF.CDFILDEST = ULT_AF.CDFILDEST AND ITEMAUTF.CDPRODUTO = :P_CDPRODUTO AND ITEMAUTF.CDMARCA = :P_CDMARCA AND ITEMAUTF.CDFILDEST = AUTOFORNE.CDFILDEST AND ITEMAUTF.NRAUTOFORN = AUTOFORNE.NRAUTOFORN AND ITEMAUTF.CDFILLANC = AUTOFORNE.CDFILLANC AND ITEMAUTF.NRORG = AUTOFORNE.NRORG AND FORNECEDOR.NRORG = FORMPGTO.NRORG AND FORMPGTO.NRORG = :P_NRORG AND AUTOFORNE.CDFORNECED = FORNECEDOR.CDFORNECED AND AUTOFORNE.CDFORMPGTO = FORMPGTO.CDFORMPGTO(+) AND NVL(AUTOFORNE.IDBAIXAAF, '0') <> '5' AND ITEMAUTF.VRPREUNIAF = ( SELECT MIN(ITM.VRPREUNIAF) FROM ITEMAUTF ITM WHERE ITM.CDFILDEST = ITEMAUTF.CDFILDEST AND ITM.NRAUTOFORN = ITEMAUTF.NRAUTOFORN AND ITM.CDPRODUTO = :P_CDPRODUTO AND ITM.CDMARCA = :P_CDMARCA ) AND AUTOFORNE.DTAUTOFORN = ( SELECT MAX(DTAUTOFORN) FROM AUTOFORNE WHERE ITEMAUTF.CDFILDEST = AUTOFORNE.CDFILDEST AND ITEMAUTF.NRAUTOFORN = AUTOFORNE.NRAUTOFORN AND ITEMAUTF.CDFILLANC = AUTOFORNE.CDFILLANC ) ORDER BY ITEMAUTF.VRPREUNIAF ASC";
        $params = [
            "P_CDCENTCMP"     => $params['CDFILCENT'],
            "P_CDPRODUTO"     => $params['CDPRODUTO'],
            "P_CDMARCA"       => $params['CDMARCA'],
            'P_NRORG'         => $this->environment->getNrorg()
        ];
        return $this->supUtilities->crud($query, $params);
    }
    
    public function getMenorPrecoEco($params){    
        $query = "SELECT ITEMCOLE.CDFORNECED, FORNECEDOR.NMFANTFORN, ITEMCOLE.VRPRECUNIITM, ITEMCOLE.VRPRECECOITM, ITEMCOLE.IDTPFOPGTCOL, ITEMCOLE.NRDIAPGFXCOL, ITEMCOLE.NRPARCPGTCOL, ITEMCOLE.IDUTDIAPGCOL, ITEMCOLE.CDFORMPGTO, FORMPGTO.NMFORMPGTO, ITEMCOLE.NRCOLEPRECO, ITEMCOLE.CDREGIAO FROM ITEMCOLE, FORMPGTO, FORNECEDOR, PARASUPRI WHERE PARASUPRI.CDFILIAL = :P_CDFILIAL AND ITEMCOLE.NRCOLEPRECO <> :P_NRCOLEPRECO AND ITEMCOLE.DTINIVALITM >= :P_DTINIVALITM AND ITEMCOLE.DTFINVALITM <= :P_DTFINVALITM AND ITEMCOLE.CDFILIAL = :P_CDFILIAL AND ITEMCOLE.CDPRODUTO = :P_CDPRODUTO AND ITEMCOLE.CDMARCA = :P_CDMARCA AND ITEMCOLE.CDREGIAO = :P_CDREGIAO AND ITEMCOLE.CDFORNECED = FORNECEDOR.CDFORNECED AND ITEMCOLE.NRORG = FORMPGTO.NRORG AND FORMPGTO.NRORG = FORNECEDOR.NRORG AND FORNECEDOR.NRORG = PARASUPRI.NRORG AND PARASUPRI.NRORG = :P_NRORG AND ITEMCOLE.CDFORMPGTO = FORMPGTO.CDFORMPGTO(+) AND NVL(ITEMCOLE.VRPRECECOITM * PARASUPRI.IDTIPOAPUR, ITEMCOLE.VRPRECUNIITM) = (SELECT NVL(MIN(ITEMCOLE.VRPRECECOITM * PARASUPRI.IDTIPOAPUR), MIN(ITEMCOLE.VRPRECUNIITM)) FROM ITEMCOLE, FORNECEDOR, PARASUPRI WHERE PARASUPRI.CDFILIAL = :P_CDFILIAL AND ITEMCOLE.CDFILIAL = :P_CDFILIAL AND ITEMCOLE.CDPRODUTO = :P_CDPRODUTO AND ITEMCOLE.CDMARCA = :P_CDMARCA AND ITEMCOLE.CDREGIAO = :P_CDREGIAO AND ITEMCOLE.DTINIVALITM >= :P_DTINIVALITM AND ITEMCOLE.DTFINVALITM <= :P_DTFINVALITM AND ITEMCOLE.NRCOLEPRECO <> :P_NRCOLEPRECO AND ITEMCOLE.CDFORNECED = FORNECEDOR.CDFORNECED AND FORNECEDOR.IDTIPOFORN = :P_IDTIPOFORN )";
        $params = [
            "P_CDFILIAL"      => $params['CDFILIAL'],
            "P_NRCOLEPRECO"   => $params['NRCOLEPRECO'],
            "P_DTINIVALITM"   => $params['DTINIVALITM'],
            "P_DTFINVALITM"   => $params['DTFINVALITM'],
            "P_CDPRODUTO"     => $params['CDPRODUTO'],
            "P_CDMARCA"       => $params['CDMARCA'],
            "P_CDREGIAO"      => $params['CDREGIAO'],
            "P_IDTIPOFORN"    => $params['IDTIPOFORN'],
            'P_NRORG'         => $this->environment->getNrorg()
        ];
        return $this->supUtilities->crud($query, $params);
    }
    
    public function getPrazoPgtoItmCol($params){    
        $query = "SELECT PZOPGTITMCOL.CDFILIAL, PZOPGTITMCOL.NRCOLEPRECO, PZOPGTITMCOL.CDFORNECED, PZOPGTITMCOL.CDPRODUTO, PZOPGTITMCOL.CDMARCA, PZOPGTITMCOL.CDREGIAO, PZOPGTITMCOL.NRDIASPGTCOL, PZOPGTITMCOL.VRPERCPGTCOL FROM PZOPGTITMCOL WHERE PZOPGTITMCOL.CDFILIAL = :P_CDFILIAL AND PZOPGTITMCOL.NRCOLEPRECO = :P_NRCOLEPRECO AND PZOPGTITMCOL.CDFORNECED = :P_CDFORNECED AND PZOPGTITMCOL.CDPRODUTO = :P_CDPRODUTO AND PZOPGTITMCOL.CDMARCA = :P_CDMARCA AND PZOPGTITMCOL.CDREGIAO = :P_CDREGIAO AND PZOPGTITMCOL.NRORG = :P_NRORG ORDER BY PZOPGTITMCOL.NRDIASPGTCOL";
        $params = [
            "P_CDFILIAL"      => $params['CDFILIAL'],
            "P_NRCOLEPRECO"   => $params['NRCOLEPRECO'],
            "P_CDFORNECED"    => $params['CDFORNECED'],
            "P_CDPRODUTO"     => $params['CDPRODUTO'],
            "P_CDMARCA"       => $params['CDMARCA'],
            "P_CDREGIAO"      => $params['CDREGIAO'],
            'P_NRORG'         => $this->environment->getNrorg()
        ];
        return $this->supUtilities->crud($query, $params);
    }
    
    public function buscaValores($params){    
        $query = "SELECT ITEMCOLE.VRPRECUNIITM, ITEMCOLE.VRPRECECOITM FROM ITEMCOLE WHERE CDREGIAO = :P_CDREGIAO AND CDFILIAL = :P_CDFILIAL AND NRCOLEPRECO = :P_NRCOLEPRECO AND CDPRODUTO = :P_CDPRODUTO AND CDMARCA = :P_CDMARCA AND CDFORNECED = :P_CDFORNECED AND NRORG = :P_NRORG";
        $params = [
            "P_CDREGIAO"      => $params['CDREGIAO'],
            "P_CDFILIAL"      => $params['CDFILIAL'],
            "P_NRCOLEPRECO"   => $params['NRCOLEPRECO'],
            "P_CDPRODUTO"     => $params['CDPRODUTO'],
            "P_CDMARCA"       => $params['CDMARCA'],
            "P_CDFORNECED"    => $params['CDFORNECED'],
            'P_NRORG'         => $this->environment->getNrorg()
        ];
        return $this->supUtilities->crud($query, $params);
    }
    
    public function getObservacao($params){    
        $query = "SELECT DSOBSITMCOL FROM ITEMCOLE WHERE CDFILIAL = :P_CDFILIAL AND NRCOLEPRECO = :P_NRCOLEPRECO AND CDREGIAO = :P_CDREGIAO AND CDFORNECED = :P_CDFORNECED AND CDPRODUTO = :P_CDPRODUTO AND CDMARCA = :P_CDMARCA AND NRORG = :P_NRORG";
        $params = [
            "P_CDFILIAL"      => $params['CDFILIAL'],
            "P_NRCOLEPRECO"   => $params['NRCOLEPRECO'],
            "P_CDREGIAO"      => $params['CDREGIAO'],
            "P_CDFORNECED"    => $params['CDFORNECED'],
            "P_CDPRODUTO"     => $params['CDPRODUTO'],
            "P_CDMARCA"       => $params['CDMARCA'],
            'P_NRORG'         => $this->environment->getNrorg()
        ];
        return $this->supUtilities->crud($query, $params);
    }
    
    public function verificaFornAtende($params){    
        $query = "SELECT CDREGIAO FROM ITEMCOLE WHERE CDFILIAL =:P_CDFILIAL AND NRCOLEPRECO =:P_NRCOLEPRECO AND CDREGIAO = :P_CDREGIAO AND CDFORNECED =:P_CDFORNECED AND CDPRODUTO =:P_CDPRODUTO AND CDMARCA =:P_CDMARCA AND NRORG =:P_NRORG AND EXISTS ( SELECT CDFORNECED, CDREGIAO FROM REFOREG WHERE REFOREG.CDFORNECED = ITEMCOLE.CDFORNECED AND REFOREG.CDREGIAO = ITEMCOLE.CDREGIAO )";
        $params = [
            "P_CDFILIAL"      => $params['CDFILIAL'],
            "P_NRCOLEPRECO"   => $params['NRCOLEPRECO'],
            "P_CDREGIAO"      => $params['CDREGIAO'],
            "P_CDFORNECED"    => $params['CDFORNECED'],
            "P_CDPRODUTO"     => $params['CDPRODUTO'],
            "P_CDMARCA"       => $params['CDMARCA'],
            'P_NRORG'         => $this->environment->getNrorg()
        ];
        return $this->supUtilities->crud($query, $params);
    }
    
    public function validaProdCmp($params){
        $query = "SELECT PRODUTO.CDPRODUTO FROM PRODUTO, PARASUPRI WHERE PARASUPRI.CDFILIAL =:P_CDFILIAL AND ( ( PRODUTO.CDARVPROD BETWEEN :P_CDPRODINI AND :P_CDPRODFIN ) OR (:P_TPRODUTO = 'T' ) ) AND ( ( PARASUPRI.IDUTLPROCMP = 'N' ) OR ( PARASUPRI.IDUTLPROCMP = 'S' AND PRODUTO.CDPRODUTO <> PRODUTO.CDPRODESTO ) ) AND PRODUTO.NRORG = PARASUPRI.NRORG AND PARASUPRI.NRORG = :P_NRORG";
        $params = [
            "P_CDFILIAL"      => $params['CDFILIAL'],
            "P_CDPRODINI"     => $params['CDPRODINI'],
            "P_CDPRODFIN"     => self::strPadRigthProd($params['CDPRODFIN']),
            "P_TPRODUTO"      => $params['TPRODUTO'],
            'P_NRORG'         => $this->environment->getNrorg()
        ];
        return $this->supUtilities->crud($query, $params);
    }
    	
    private function strPadRigthProd($value)
    {
	    return str_pad($value, 20, 'Z', STR_PAD_RIGHT);
	}

    
    public function getSolicita($params){
        $query = "SELECT SUM(PRODACOMPRAR.QTACOMPRAR) AS QTACOMPRAR FROM PRODACOMPRAR, FILIAL, PARASUPRI, PRODUTO WHERE ( ( PRODUTO.CDARVPROD BETWEEN :P_CDPRODINI AND :P_CDPRODFIN ) OR (:P_TPRODUTO = 'T' ) ) AND ( PRODACOMPRAR.DTPROGENTR BETWEEN :P_DTINI AND :P_DTFIN ) AND PRODACOMPRAR.CDFILIAL = FILIAL.CDFILIAL AND ( ( FILIAL.CDFILCENTCMP =:P_CDFILIAL ) OR ( FILIAL.CDFILIAL =:P_CDFILIAL AND FILIAL.CDFILCENTCMP IS NULL ) ) AND ( INSTR(:P_CDREGIAO,FILIAL.CDREGIAO) <> 0 OR (:P_CDREGIAO = 'T' ) ) AND PRODUTO.CDPRODUTO = PRODACOMPRAR.CDPRODUTO AND PARASUPRI.CDFILIAL =:P_CDFILIAL AND ( ( PARASUPRI.IDUTLPROCMP = 'N' ) OR PARASUPRI.IDUTLPROCMP = 'S' AND PRODUTO.CDPRODUTO <> PRODUTO.CDPRODESTO ) AND FILIAL.NRORG = PARASUPRI.NRORG AND PARASUPRI.NRORG = PRODUTO.NRORG AND PRODUTO.NRORG =:P_NRORG";
        $params = [
            "P_CDFILIAL"      => $params['CDFILIAL'],
            "P_CDPRODINI"     => $params['CDPRODINI'],
            "P_CDPRODFIN"     => self::strPadRigthProd($params['CDPRODFIN']),
            "P_TPRODUTO"      => $params['TPRODUTO'],
            "P_CDREGIAO"      => $params['CDREGIAO'],
            "P_DTINI"         => $params['DTINIVALCOL'],
            "P_DTFIN"         => $params['DTFINVALCOL'],
            'P_NRORG'         => $this->environment->getNrorg()
        ];
        return $this->supUtilities->crud($query, $params);
    }
    
    public function getFornProdCmp($params){
        $query = "SELECT PRODFORNCOMP.CDPRODUTO FROM PRODFORNCOMP, PRODUTO PF, PARASUPRI WHERE ( :P_CDFORNECED LIKE '%' || PRODFORNCOMP.CDFORNECED || '%' OR :P_CDFORNECED = 'T' ) AND ( PF.CDARVPROD BETWEEN :P_CDPRODINI AND :P_CDPRODFIN OR :P_TPRODUTO = 'T' ) AND ( PRODFORNCOMP.CDPRODCOMPL = PF.CDPRODUTO ) AND ( PARASUPRI.CDFILIAL =:P_CDFILIAL ) AND ( ( PARASUPRI.IDUTLPROCMP = 'N' ) OR ( PARASUPRI.IDUTLPROCMP = 'S' AND PF.CDPRODUTO <> PF.CDPRODESTO ) ) AND PRODFORNCOMP.NRORG = PF.NRORG AND PF.NRORG = PARASUPRI.NRORG AND PARASUPRI.NRORG =:P_NRORG";
        $params = [
            "P_CDFILIAL"      => $params['CDFILIAL'],
            "P_CDFORNECED"    => $params['CDFORNECED'],
            "P_CDPRODINI"     => $params['CDPRODINI'],
            "P_CDPRODFIN"     => self::strPadRigthProd($params['CDPRODFIN']),
            "P_TPRODUTO"      => $params['TPRODUTO'],
            'P_NRORG'         => $this->environment->getNrorg()
        ];
        return $this->supUtilities->crud($query, $params);
    }
    
    public function getProdComp($params){
        $query = "SELECT PRODCOMP.CDOPERADOR FROM PARASUPRI, PRODUTO PC, PRODUTO P, PRODCOMP WHERE ( PRODCOMP.CDOPERADOR =:P_CDOPERADOR ) AND ( P.CDARVPROD BETWEEN :P_CDPRODINI AND :P_CDPRODFIN OR :P_TPRODUTO = 'T' ) AND ( P.CDARVPROD LIKE PC.CDARVPROD || '%' ) AND ( PRODCOMP.CDPRODUTO = PC.CDPRODUTO ) AND ( PARASUPRI.CDFILIAL =:P_CDFILIAL ) AND ( ( PARASUPRI.IDUTLPROCMP = 'N' ) OR ( PARASUPRI.IDUTLPROCMP = 'S' AND PC.CDPRODUTO <> PC.CDPRODESTO ) ) AND PARASUPRI.NRORG = PC.NRORG AND PC.NRORG = P.NRORG AND P.NRORG = PRODCOMP.NRORG AND PRODCOMP.NRORG = :P_NRORG";
        $params = [
            "P_CDFILIAL"      => $params['CDFILIAL'],
            "P_CDOPERADOR"    => $params['CDOPERINCL'],
            "P_CDPRODINI"     => $params['CDPRODINI'],
            "P_CDPRODFIN"     => self::strPadRigthProd($params['CDPRODFIN']),
            "P_TPRODUTO"      => $params['TPRODUTO'],
            'P_NRORG'         => $this->environment->getNrorg()
        ];
        return $this->supUtilities->crud($query, $params);
    }
    
    public function getMarcProd($params){
        $query = "SELECT MARCPROD.CDMARCA FROM MARCPROD, PRODUTO, PRODUTO PMARC WHERE ( INSTR(:P_CDMARCA,MARCPROD.CDMARCA) <> 0 OR :P_CDMARCA = 'T' ) AND ( PRODUTO.CDARVPROD LIKE PMARC.CDARVPROD || '%' ) AND ( PMARC.CDPRODUTO = MARCPROD.CDPRODUTO ) AND ( ( PRODUTO.CDARVPROD BETWEEN :P_CDPRODINI AND :P_CDPRODFIN ) OR (:P_TPRODUTO = 'T' ) ) AND MARCPROD.NRORG = PRODUTO.NRORG AND PRODUTO.NRORG =:P_NRORG";
        $params = [
            "P_CDPRODINI"     => $params['CDPRODINI'],
            "P_CDPRODFIN"     => self::strPadRigthProd($params['CDPRODFIN']),
            "P_TPRODUTO"      => $params['TPRODUTO'],
            "P_CDMARCA"       => $params['CDMARCA'],
            'P_NRORG'         => $this->environment->getNrorg()
        ];
        return $this->supUtilities->crud($query, $params);
    }
    
    public function validaQualificaMarcProd($params){
        $query = "SELECT CDMARCA FROM MARCA WHERE ( INSTR(:P_CDMARCA,MARCA.CDMARCA) <> 0 OR :P_CDMARCA = 'T' ) AND NOT EXISTS ( SELECT QFOMARPROD.CDMARCA FROM QFOMARPROD, PRODUTO, PARASUPRI WHERE ( QFOMARPROD.CDPRODUTO = PRODUTO.CDPRODUTO ) AND ( MARCA.CDMARCA = QFOMARPROD.CDMARCA ) AND ( ( PRODUTO.CDARVPROD BETWEEN :P_CDPRODINI AND :P_CDPRODFIN ) OR (:P_TPRODUTO = 'T' ) ) AND ( PARASUPRI.CDFILIAL =:P_CDFILIAL ) AND ( ( QFOMARPROD.IDAPRMARPRO = 'N' AND QFOMARPROD.IDAUTMARPRO = 'N' ) OR ( ( QFOMARPROD.IDAPRMARPRO = 'S' ) AND ( ( PARASUPRI.IDULTIDTVALQ = 'M' AND TO_DATE(:P_DTCOL) > LAST_DAY(QFOMARPROD.DTVALAPROVA) ) OR ( PARASUPRI.IDULTIDTVALQ = 'D' AND TO_DATE(:P_DTCOL) > QFOMARPROD.DTVALAPROVA ) ) ) ) AND QFOMARPROD.NRORG = PRODUTO.NRORG AND PRODUTO.NRORG = PARASUPRI.NRORG AND PARASUPRI.NRORG = MARCA.NRORG ) AND EXISTS ( SELECT QFOMARPROD.CDMARCA FROM QFOMARPROD, PRODUTO WHERE ( ( PRODUTO.CDARVPROD BETWEEN :P_CDPRODINI AND :P_CDPRODFIN ) OR (:P_TPRODUTO = 'T' ) ) AND QFOMARPROD.CDMARCA = MARCA.CDMARCA AND QFOMARPROD.CDPRODUTO = PRODUTO.CDPRODUTO AND QFOMARPROD.NRORG = PRODUTO.NRORG AND PRODUTO.NRORG = MARCA.NRORG UNION SELECT PARASUPRI.CDMARCA FROM PARASUPRI WHERE PARASUPRI.CDFILIAL =:P_CDFILIAL AND ( PARASUPRI.IDMARSEMQUA = 'N' OR PARASUPRI.IDMARSEMQUA IS NULL ) AND PARASUPRI.NRORG = MARCA.NRORG ) AND MARCA.NRORG = :P_NRORG";
        $params = [
            "P_CDFILIAL"      => $params['CDFILIAL'],
            "P_CDPRODINI"     => $params['CDPRODINI'],
            "P_CDPRODFIN"     => self::strPadRigthProd($params['CDPRODFIN']),
            "P_TPRODUTO"      => $params['TPRODUTO'],
            "P_CDMARCA"       => $params['CDMARCA'],
            "P_DTCOL"         => $params['DTCOLEPREC'],
            'P_NRORG'         => $this->environment->getNrorg()
        ];
        return $this->supUtilities->crud($query, $params);
    }
    
    public function validaQfoProdForn($params){
        $query = "SELECT FORNECEDOR.CDFORNECED FROM FORNECEDOR, PRODUTO WHERE ( ( PRODUTO.CDARVPROD BETWEEN :P_CDPRODINI AND :P_CDPRODFIN ) OR :P_TPRODUTO = 'T' ) AND ( PRODUTO.CDPRODESTO IS NOT NULL ) AND ( INSTR(:P_CDFORNECED,FORNECEDOR.CDFORNECED) <> 0 OR :P_CDFORNECED = 'T' ) AND NOT EXISTS ( SELECT QFOPRODFORN.CDPRODUTO FROM QFOPRODFORN, PARASUPRI WHERE PARASUPRI.CDFILIAL =:P_CDFILIAL AND ( QFOPRODFORN.CDPRODUTO = PRODUTO.CDPRODUTO ) AND ( QFOPRODFORN.CDFORNECED = FORNECEDOR.CDFORNECED ) AND ( ( QFOPRODFORN.IDFORNAPROV = 'N' AND QFOPRODFORN.IDFORNAUTOR = 'N' ) OR ( ( QFOPRODFORN.IDFORNAPROV = 'S' ) AND ( ( PARASUPRI.IDULTIDTVALQ = 'M' AND :P_DTCOL > LAST_DAY(QFOPRODFORN.DTVALPRFORN) ) OR ( PARASUPRI.IDULTIDTVALQ = 'D' AND :P_DTCOL > QFOPRODFORN.DTVALPRFORN ) ) ) ) AND QFOPRODFORN.NRORG = PARASUPRI.NRORG AND PARASUPRI.NRORG = :P_NRORG ) AND FORNECEDOR.NRORG = PRODUTO.NRORG AND PRODUTO.NRORG = :P_NRORG";
        $params = [
            "P_CDFILIAL"      => $params['CDFILIAL'],
            "P_CDFORNECED"    => $params['CDFORNECED'],
            "P_CDPRODINI"     => $params['CDPRODINI'],
            "P_CDPRODFIN"     => self::strPadRigthProd($params['CDPRODFIN']),
            "P_TPRODUTO"      => $params['TPRODUTO'],
            "P_DTCOL"         => $params['DTCOLEPREC'],
            'P_NRORG'         => $this->environment->getNrorg()
        ];
        return $this->supUtilities->crud($query, $params);
    }
    
    public function getRegFoReg($params){
        $query = "SELECT CDFORNECED FROM FORNECEDOR, REGIAO WHERE ( INSTR(:P_CDFORNECED,CDFORNECED) <> 0 OR :P_CDFORNECED = 'T' ) AND ( INSTR(:P_CDREGIAO,CDREGIAO) <> 0 OR :P_CDREGIAO = 'T' ) AND NOT EXISTS ( SELECT CDREGIAO FROM REFOREG WHERE REFOREG.CDFORNECED = FORNECEDOR.CDFORNECED AND REFOREG.CDREGIAO = REGIAO.CDREGIAO ) AND FORNECEDOR.NRORG = REGIAO.NRORG AND REGIAO.NRORG = :P_NRORG";
        $params = [
            "P_CDFORNECED"    => $params['CDFORNECED'],
            "P_CDREGIAO"      => $params['CDREGIAO'],
            'P_NRORG'         => $this->environment->getNrorg()
        ];
        return $this->supUtilities->crud($query, $params);
    }
    
    public function validaFiliCor($params){
        $query = "SELECT PARAMPRODSUP.CDREGIAO, PARAMPRODSUP.CDPRODUTO FROM PARAMPRODSUP, PRODUTO, FILIAL WHERE FILIAL.CDFILIAL =:P_CDFILIPROC AND FILIAL.IDFUNCAOFILI = 'C' AND FILIAL.CDFILIAL = PARAMPRODSUP.CDFILICTRL AND PRODUTO.CDPRODUTO = PARAMPRODSUP.CDPRODUTO AND ( ( PRODUTO.CDARVPROD BETWEEN :P_CDPRODINI AND :P_CDPRODFIN ) OR (:P_TPRODUTO = 'T' ) ) AND ( INSTR(:P_CDREGIAO,PARAMPRODSUP.CDREGIAO) <> 0 OR (:P_CDREGIAO = 'T' ) ) AND PARAMPRODSUP.IDFILINEGCOR = 'S' AND PARAMPRODSUP.NRORG = PRODUTO.NRORG AND PRODUTO.NRORG = FILIAL.NRORG AND FILIAL.NRORG =:P_NRORG";
        $params = [
            "P_CDFILIPROC"    => $params['CDFILIPROC'],
            "P_CDPRODINI"     => $params['CDPRODINI'],
            "P_CDPRODFIN"     => self::strPadRigthProd($params['CDPRODFIN']),
            "P_TPRODUTO"      => $params['TPRODUTO'],
            "P_CDREGIAO"      => $params['CDREGIAO'],
            'P_NRORG'         => $this->environment->getNrorg()
        ];
        return $this->supUtilities->crud($query, $params);
    }
    
    public function getProdInativo($params){
        $query = "SELECT PRODUTO.CDPRODUTO FROM PRODUTO WHERE ( PRODUTO.IDPRODATIVPR = 'N' ) AND ( ( PRODUTO.CDARVPROD BETWEEN :P_CDPRODINI AND :P_CDPRODFIN ) OR (:P_TPRODUTO = 'T' ) ) AND ( PRODUTO.NRORG =:P_NRORG )";
        $params = [
            "P_CDPRODINI"     => $params['CDPRODINI'],
            "P_CDPRODFIN"     => self::strPadRigthProd($params['CDPRODFIN']),
            "P_TPRODUTO"      => $params['TPRODUTO'],
            'P_NRORG'         => $this->environment->getNrorg()
        ];
        return $this->supUtilities->crud($query, $params);
    }
    
    public function getDtQualifForn($params){
        $query = "SELECT FORNECEDOR.CDFORNECED FROM QUALIFICACAO, PARASUPRI, FORNECEDOR WHERE ( INSTR(:P_CDFORNECED,FORNECEDOR.CDFORNECED) <> 0 OR :P_CDFORNECED = 'T' ) AND ( FORNECEDOR.CDQUALIF = QUALIFICACAO.CDQUALIF ) AND ( PARASUPRI.CDFILIAL =:P_CDFILIAL ) AND ( FORNECEDOR.IDFORNAUTO = 'N' ) AND ( ( QUALIFICACAO.IDPERCOTCMP = '2' ) OR ( ( PARASUPRI.IDULTIDTVALQ = 'D' AND :P_DTFIN > FORNECEDOR.DTVALQUALIF ) OR ( PARASUPRI.IDULTIDTVALQ = 'M' AND :P_DTFIN > LAST_DAY(FORNECEDOR.DTVALQUALIF) ) ) ) AND QUALIFICACAO.NRORG = PARASUPRI.NRORG AND PARASUPRI.NRORG = FORNECEDOR.NRORG AND FORNECEDOR.NRORG = :P_NRORG";
        $params = [
            "P_CDFILIAL"      => $params['CDFILIAL'],
            "P_DTFIN"         => $params['DTFINVALCOL'],
            "P_CDFORNECED"    => $params['CDFORNECED'],
            'P_NRORG'         => $this->environment->getNrorg()
        ];
        return $this->supUtilities->crud($query, $params);
    }
    
    public function getFornecedorRelacionado($row){
        $query = "SELECT DISTINCT CDFORNECED FROM PRODFORN WHERE CDPRODUTO BETWEEN :P_CDPRODINI AND :P_CDPRODFIN AND NRORG = :P_NRORG";
        $params = [
            "P_CDPRODINI"     => $row['CDPRODINI'],
            "P_CDPRODFIN"     => self::strPadRigthProd($row['CDPRODFIN']),
            'P_NRORG'         => $this->environment->getNrorg()
        ];
        return $this->supUtilities->crud($query, $params);
    }
    
    public function validaOperadorComprador($row){
        $query = "SELECT COMPRADOR.CDOPERADOR, OPERADOR.NMOPERADOR, OPERADOR.NRORG, NVL(COMPROPER.IDEXSENHCMPOPER, 'S') AS IDEXSENHCMPOPER
                    FROM COMPRADOR INNER JOIN OPERADOR ON COMPRADOR.CDOPERADOR = OPERADOR.CDOPERADOR 
                                                      AND COMPRADOR.NRORG = OPERADOR.NRORG
                                    LEFT JOIN COMPROPER ON COMPRADOR.CDOPERADOR = COMPROPER.CDOPERADOR
                                                       AND COMPRADOR.NRORG = COMPROPER.NRORG                                    
                   WHERE ((COMPRADOR.CDOPERADOR = :P_CDOPERADOR) OR (:P_CDOPERADOR = 'T')) 
                     AND OPERADOR.NRORG = :P_NRORG";
        $params = [
            "P_CDOPERADOR"    => $this->environment->getCdoperador(),
            'P_NRORG'         => $this->environment->getNrorg()
        ];
        return $this->supUtilities->crud($query, $params);
    }
    
    public function getParcelas($params)
    {
        $query = "SELECT ITEMCOLE.CDFILIAL, ITEMCOLE.NRCOLEPRECO, ITEMCOLE.CDFORNECED, ITEMCOLE.CDPRODUTO, ITEMCOLE.CDMARCA, ITEMCOLE.CDREGIAO, ITEMCOLE.NRORG, PRAZOPGTFORN.NRDIASPGTOFO, PRAZOPGTFORN.VRPERCPGTOFO FROM ITEMCOLE, PRAZOPGTFORN WHERE ITEMCOLE.IDTPFOPGTCOL IS NOT NULL AND ITEMCOLE.CDFILIAL = :P_CDFILIAL AND ITEMCOLE.NRCOLEPRECO = :P_NRCOLEPRECO AND ITEMCOLE.CDFORNECED = :P_CDFORNECED AND ITEMCOLE.CDPRODUTO = :P_CDPRODUTO AND ITEMCOLE.CDMARCA = :P_CDMARCA AND ITEMCOLE.CDREGIAO = :P_CDREGIAO AND ITEMCOLE.NRORG = PRAZOPGTFORN.NRORG AND PRAZOPGTFORN.NRORG = :P_NRORG AND PRAZOPGTFORN.CDFORNECED = ITEMCOLE.CDFORNECED";
        $params = [
            "P_CDFILIAL"    => $params['CDFILIAL'],
            "P_NRCOLEPRECO" => $params['NRCOLEPRECO'],    
            "P_CDPRODUTO"   => $params['CDPRODUTO'],    
            "P_CDMARCA"     => $params['CDMARCA'],    
            "P_CDFORNECED"  => $params['CDFORNECED'],    
            "P_CDREGIAO"    => $params['CDREGIAO'],    
            'P_NRORG'       => $this->environment->getNrorg()
        ];
        return $this->supUtilities->crud($query, $params);
    }
     
    /********************
     *      INSERTS     *
     * ******************/
     
     
    public function salvaCotacao($row, $NRCOLEPREC)
    {
        try {
            $query = "INSERT INTO COLETAPR ( CDFILIAL, NRCOLEPRECO, NRSEMAINIC, NRSEMAFINAL, DTINIVALCOL, DTFINVALCOL, CDOPERADOR, DTCOLEPREC, IDDISMONTLIS, CDOPERINCL, DTINCLCOLET, DSOBSCOTPREC, IDSITUCOLETA, NRCOMPRA, NRORG, CDTPINDICECOL, IDPERVALORCOL, CDINDICECOL, DTREFINDCOL ) VALUES ( :P_CDFILIAL, :P_NRCOLEPRECO, :P_NRSEMAINIC , :P_NRSEMAFINAL, :P_DTINIVALCOL, :P_DTFINVALCOL, :P_CDOPERADOR, :P_DTCOLEPREC, :P_IDDISMONTLIS, :P_CDOPERINCL, SYSDATE, :P_DSOBSCOTPREC, :P_IDSITUCOLETA, :P_NRCOMPRA, :P_NRORG, :P_CDTPINDICE, :P_IDPERVALOR, :P_CDINDICE, :P_DTREFIND)";
            $param = [
                'P_CDFILIAL'     => $row['CDFILIAL'],
                'P_NRCOLEPRECO'  => $NRCOLEPREC,
                'P_NRSEMAINIC'   => $row['NRSEMAINIC'],
                'P_NRSEMAFINAL'  => $row['NRSEMAFINAL'],
                'P_DTINIVALCOL'  => $row['DTINIVALCOL'],
                'P_DTFINVALCOL'  => $row['DTFINVALCOL'],
                'P_CDOPERADOR'   => $row['CDCOMPRADOR'],
                'P_DTCOLEPREC'   => $row['DTCOLEPREC'],
                'P_IDDISMONTLIS' => $row['IDDISMONTLIS'],
                'P_CDOPERINCL'   => $row['CDOPERINCL'],
                'P_DSOBSCOTPREC' => $row['DSOBSCOTPREC'],
                'P_IDSITUCOLETA' => $row['IDSITUCOLETA'],
                'P_NRCOMPRA'     => $row['NRCOMPRA'],
                'P_NRORG'        => $this->environment->getNrorg(),
                'P_CDTPINDICE'   => $row['CDTPINDICE'],
                'P_IDPERVALOR'   => $row['IDPERVALOR'],
                'P_CDINDICE'     => $row['CDINDICE'],
                'P_DTREFIND'     => $row['DTREFIND']    
            ];
            return $this->supUtilities->crud($query, $param);
        }
        catch (Exception $e) {
            throw new \Exception('Erro ao inserir cotação!');
        }
    }
    
    private function calculaVrIndice($params, $row){
        try {
            $qryIndice = "SELECT VRINDICE 
                            FROM MOVINDICE
                           WHERE CDTPINDICE = :P_CDTPINDICE
                             AND IDPERVALOR = :P_IDPERVALOR
                             AND CDINDICE = :P_CDINDICE
                             AND DTREFIND = :P_DTREFINDCOL
                             AND NRORG = :P_NRORG";
            $parIndice = [
                'P_CDTPINDICE' => $row['CDTPINDICECOL'],
                'P_IDPERVALOR' => $row['IDPERVALORCOL'],
                'P_CDINDICE'   => $row['CDINDICECOL'],
                'P_DTREFINDCOL'=> $row['DTREFINDCOL'],
                'P_NRORG'      => $this->environment->getNrorg()
            ];
            $vrIndice = $this->supUtilities->crud($qryIndice, $parIndice); 
            if($vrIndice){
                if($row['IDPERVALORCOL'] == '2'){
                    $params['VRPRECUNIITM'] = $params['VRINDICEITMCOL'] * $vrIndice[0]['VRINDICE'];
                } else {
                    $params['VRPRECUNIITM'] = $params['VRINDICEITMCOL'] * ($vrIndice[0]['VRINDICE']/100);
                }
            }
            return $params['VRPRECUNIITM'];
        } catch (Exception $e) {
            throw new \Exception('Erro ao atualizar os registros!');
        }
    }
    
    public function insertItemCole($params, $row)
    {
        try {
            if($row['IDUTINDICECOL'] == 'S'){
                $params['VRINDICEITMCOL'] = $params['VRPRECUNIITM'];
                $params['VRPRECUNIITM'] = $this->calculaVrIndice($params, $row);
                
                $query = "INSERT INTO ITEMCOLE (CDFILIAL, NRCOLEPRECO, CDPRODUTO, CDMARCA, CDFORNECED, CDREGIAO, DTINIVALITM, DTFINVALITM, VRPRECUNIITM, VRINDICEITMCOL, VRPERDESITM, IDTIPFREITM, IDITMCOTPREC, IDATUREGIINC, CDOPERINCL, DTINCLITMCOL, IDATUFORNINC, NRSEQPRELIST, IDITMLISPREC, IDPRECALT, IDITVALDIFLISTA, NRORG) SELECT :CDFILIAL, :NRCOLEPRECO, :CDPRODUTO, :CDMARCA, :CDFORNECED, :CDREGIAO, COLETAPR.DTINIVALCOL, COLETAPR.DTFINVALCOL, :VRPRECUNIITM, :VRINDICEITMCOL, DECODE(FORNECEDOR.IDTPDESCFORN, 'C', FORNECEDOR.VRPEDESCFORN, 0), FORNECEDOR.IDTPFRETFORN, 'S', :P_IDATUREG, :P_CDOPERINCL, SYSDATE, :P_IDATUFORN, :P_NRSEQPRELIST, DECODE(:P_NRSEQPRELIST, NULL, 'N', 'S'), :P_IDPRECALT, :P_IDITVALDIFLISTA, :P_NRORG FROM COLETAPR, FORNECEDOR WHERE COLETAPR.CDFILIAL = :CDFILIAL AND COLETAPR.NRCOLEPRECO = :NRCOLEPRECO AND FORNECEDOR.CDFORNECED = :CDFORNECED AND COLETAPR.NRORG = FORNECEDOR.NRORG AND FORNECEDOR.NRORG = :P_NRORG AND NOT EXISTS (SELECT CDFORNECED, CDREGIAO FROM REFOREG WHERE REFOREG.CDFORNECED = :CDFORNECED AND REFOREG.CDREGIAO = :CDREGIAO) AND NOT EXISTS (SELECT CDFILIAL FROM ITEMCOLE WHERE ITEMCOLE.CDFILIAL = :CDFILIAL AND ITEMCOLE.NRCOLEPRECO = :NRCOLEPRECO AND ITEMCOLE.CDREGIAO = :CDREGIAO AND ITEMCOLE.CDPRODUTO = :CDPRODUTO AND ITEMCOLE.CDMARCA = :CDMARCA AND ITEMCOLE.CDFORNECED = :CDFORNECED)";
                $param = [
                    'CDFILIAL'          => $params['CDFILIAL'],
                    'NRCOLEPRECO'       => $params['NRCOLEPRECO'],
                    'CDPRODUTO'         => $params['CDPRODUTO'],
                    'CDMARCA'           => $params['CDMARCA'],
                    'CDFORNECED'        => $params['CDFORNECED'],
                    'CDREGIAO'          => $params['CDREGIAO'],
                    'VRPRECUNIITM'      => $params['VRPRECUNIITM'],
                    'VRINDICEITMCOL'    => $params['VRINDICEITMCOL'],
                    'P_IDATUREG'        => $params['IDATUREG'],
                    'P_CDOPERINCL'      => $params['CDOPERINCL'],
                    'P_IDATUFORN'       => $params['IDATUFORN'],
                    'P_NRSEQPRELIST'    => $params['NRSEQPRELIST'],
                    'P_IDPRECALT'       => $params['IDPRECALT'],
                    'P_IDITVALDIFLISTA' => $params['IDITVALDIFLISTA'],
                    'P_NRORG'           => $this->environment->getNrorg()
                ];
            } else {
                $query = "INSERT INTO ITEMCOLE (CDFILIAL, NRCOLEPRECO, CDPRODUTO, CDMARCA, CDFORNECED, CDREGIAO, DTINIVALITM, DTFINVALITM, VRPRECUNIITM, VRPERDESITM, IDTIPFREITM, IDITMCOTPREC, IDATUREGIINC, CDOPERINCL, DTINCLITMCOL, IDATUFORNINC, NRSEQPRELIST, IDITMLISPREC, IDPRECALT, IDITVALDIFLISTA, NRORG) SELECT :CDFILIAL, :NRCOLEPRECO, :CDPRODUTO, :CDMARCA, :CDFORNECED, :CDREGIAO, COLETAPR.DTINIVALCOL, COLETAPR.DTFINVALCOL, :VRPRECUNIITM, DECODE(FORNECEDOR.IDTPDESCFORN, 'C', FORNECEDOR.VRPEDESCFORN, 0), FORNECEDOR.IDTPFRETFORN, 'S', :P_IDATUREG, :P_CDOPERINCL, SYSDATE, :P_IDATUFORN, :P_NRSEQPRELIST, DECODE(:P_NRSEQPRELIST, NULL, 'N', 'S'), :P_IDPRECALT, :P_IDITVALDIFLISTA, :P_NRORG FROM COLETAPR, FORNECEDOR WHERE COLETAPR.CDFILIAL = :CDFILIAL AND COLETAPR.NRCOLEPRECO = :NRCOLEPRECO AND FORNECEDOR.CDFORNECED = :CDFORNECED AND COLETAPR.NRORG = FORNECEDOR.NRORG AND FORNECEDOR.NRORG = :P_NRORG AND NOT EXISTS (SELECT CDFORNECED, CDREGIAO FROM REFOREG WHERE REFOREG.CDFORNECED = :CDFORNECED AND REFOREG.CDREGIAO = :CDREGIAO) AND NOT EXISTS (SELECT CDFILIAL FROM ITEMCOLE WHERE ITEMCOLE.CDFILIAL = :CDFILIAL AND ITEMCOLE.NRCOLEPRECO = :NRCOLEPRECO AND ITEMCOLE.CDREGIAO = :CDREGIAO AND ITEMCOLE.CDPRODUTO = :CDPRODUTO AND ITEMCOLE.CDMARCA = :CDMARCA AND ITEMCOLE.CDFORNECED = :CDFORNECED)";
                $param = [
                    'CDFILIAL'          => $params['CDFILIAL'],
                    'NRCOLEPRECO'       => $params['NRCOLEPRECO'],
                    'CDPRODUTO'         => $params['CDPRODUTO'],
                    'CDMARCA'           => $params['CDMARCA'],
                    'CDFORNECED'        => $params['CDFORNECED'],
                    'CDREGIAO'          => $params['CDREGIAO'],
                    'VRPRECUNIITM'      => $params['VRPRECUNIITM'],
                    'P_IDATUREG'        => $params['IDATUREG'],
                    'P_CDOPERINCL'      => $params['CDOPERINCL'],
                    'P_IDATUFORN'       => $params['IDATUFORN'],
                    'P_NRSEQPRELIST'    => $params['NRSEQPRELIST'],
                    'P_IDPRECALT'       => $params['IDPRECALT'],
                    'P_IDITVALDIFLISTA' => $params['IDITVALDIFLISTA'],
                    'P_NRORG'           => $this->environment->getNrorg()
                ];   
            }
            
            $this->supUtilities->crud($query, $param);
        }
        catch (Exception $e) {
            throw new \Exception('Erro ao inserir cotação!');
        }
    }
    
    public function insertItemColRegiao($params, $row){
        try {
            if($row['IDUTINDICECOL'] == 'S'){
                
                //$params['VRINDICEITMCOL'] = $params['VRINDICEITMCOL'];
                //$params['VRPRECUNIITM'] = $this->calculaVrIndice($params, $row);
                
                $query = "INSERT INTO ITEMCOLE (CDFILIAL, NRCOLEPRECO, CDPRODUTO, CDMARCA, CDFORNECED, CDREGIAO, DTINIVALITM, DTFINVALITM, VRPRECUNIITM, VRINDICEITMCOL, VRPERDESITM, IDTIPFREITM, IDITMCOTPREC, IDATUREGIINC, CDOPERINCL, DTINCLITMCOL, IDATUFORNINC, NRSEQPRELIST, IDITMLISPREC, IDPRECALT, IDITVALDIFLISTA, NRORG) SELECT :CDFILIAL, :NRCOLEPRECO, :CDPRODUTO, :CDMARCA, :CDFORNECED, :CDREGIAO, COLETAPR.DTINIVALCOL, COLETAPR.DTFINVALCOL, :VRPRECUNIITM, :VRINDICEITMCOL, DECODE(FORNECEDOR.IDTPDESCFORN, 'C', FORNECEDOR.VRPEDESCFORN, 0), FORNECEDOR.IDTPFRETFORN, 'S', :P_IDATUREG, :P_CDOPERINCL, SYSDATE, :P_IDATUFORN, :P_NRSEQPRELIST, DECODE(:P_NRSEQPRELIST, NULL, 'N', 'S'), :P_IDPRECALT, :P_IDITVALDIFLISTA, :P_NRORG FROM COLETAPR, FORNECEDOR WHERE COLETAPR.CDFILIAL = :CDFILIAL AND COLETAPR.NRCOLEPRECO = :NRCOLEPRECO AND FORNECEDOR.CDFORNECED = :CDFORNECED AND COLETAPR.NRORG = FORNECEDOR.NRORG AND FORNECEDOR.NRORG = :P_NRORG AND NOT EXISTS (SELECT CDFORNECED, CDREGIAO FROM REFOREG WHERE REFOREG.CDFORNECED = :CDFORNECED AND REFOREG.CDREGIAO = :CDREGIAO) AND NOT EXISTS (SELECT CDFILIAL FROM ITEMCOLE WHERE ITEMCOLE.CDFILIAL = :CDFILIAL AND ITEMCOLE.NRCOLEPRECO = :NRCOLEPRECO AND ITEMCOLE.CDREGIAO = :CDREGIAO AND ITEMCOLE.CDPRODUTO = :CDPRODUTO AND ITEMCOLE.CDMARCA = :CDMARCA AND ITEMCOLE.CDFORNECED = :CDFORNECED)";
                $param = [
                    'CDFILIAL'          => $params['CDFILIAL'],
                    'NRCOLEPRECO'       => $params['NRCOLEPRECO'],
                    'CDPRODUTO'         => $params['CDPRODUTO'],
                    'CDMARCA'           => $params['CDMARCA'],
                    'CDFORNECED'        => $params['CDFORNECED'],
                    'CDREGIAO'          => $params['CDREGIAO'],
                    'VRPRECUNIITM'      => $params['VRPRECUNIITM'],
                    'VRINDICEITMCOL'    => $params['VRINDICEITMCOL'],
                    'P_IDATUREG'        => $params['IDATUREG'],
                    'P_CDOPERINCL'      => $params['CDOPERINCL'],
                    'P_IDATUFORN'       => $params['IDATUFORN'],
                    'P_NRSEQPRELIST'    => $params['NRSEQPRELIST'],
                    'P_IDPRECALT'       => $params['IDPRECALT'],
                    'P_IDITVALDIFLISTA' => $params['IDITVALDIFLISTA'],
                    'P_NRORG'           => $this->environment->getNrorg()
                ];
            } else {
                $query = "INSERT INTO ITEMCOLE (CDFILIAL, NRCOLEPRECO, CDPRODUTO, CDMARCA, CDFORNECED, CDREGIAO, DTINIVALITM, DTFINVALITM, VRPRECUNIITM, VRPERDESITM, IDTIPFREITM, IDITMCOTPREC, IDATUREGIINC, CDOPERINCL, DTINCLITMCOL, IDATUFORNINC, NRSEQPRELIST, IDITMLISPREC, IDPRECALT, IDITVALDIFLISTA, NRORG) SELECT :CDFILIAL, :NRCOLEPRECO, :CDPRODUTO, :CDMARCA, :CDFORNECED, :CDREGIAO, COLETAPR.DTINIVALCOL, COLETAPR.DTFINVALCOL, :VRPRECUNIITM, DECODE(FORNECEDOR.IDTPDESCFORN, 'C', FORNECEDOR.VRPEDESCFORN, 0), FORNECEDOR.IDTPFRETFORN, 'S', :P_IDATUREG, :P_CDOPERINCL, SYSDATE, :P_IDATUFORN, :P_NRSEQPRELIST, DECODE(:P_NRSEQPRELIST, NULL, 'N', 'S'), :P_IDPRECALT, :P_IDITVALDIFLISTA, :P_NRORG FROM COLETAPR, FORNECEDOR WHERE COLETAPR.CDFILIAL = :CDFILIAL AND COLETAPR.NRCOLEPRECO = :NRCOLEPRECO AND FORNECEDOR.CDFORNECED = :CDFORNECED AND COLETAPR.NRORG = FORNECEDOR.NRORG AND FORNECEDOR.NRORG = :P_NRORG AND NOT EXISTS (SELECT CDFORNECED, CDREGIAO FROM REFOREG WHERE REFOREG.CDFORNECED = :CDFORNECED AND REFOREG.CDREGIAO = :CDREGIAO) AND NOT EXISTS (SELECT CDFILIAL FROM ITEMCOLE WHERE ITEMCOLE.CDFILIAL = :CDFILIAL AND ITEMCOLE.NRCOLEPRECO = :NRCOLEPRECO AND ITEMCOLE.CDREGIAO = :CDREGIAO AND ITEMCOLE.CDPRODUTO = :CDPRODUTO AND ITEMCOLE.CDMARCA = :CDMARCA AND ITEMCOLE.CDFORNECED = :CDFORNECED)";
                $param = [
                    'CDFILIAL'          => $params['CDFILIAL'],
                    'NRCOLEPRECO'       => $params['NRCOLEPRECO'],
                    'CDPRODUTO'         => $params['CDPRODUTO'],
                    'CDMARCA'           => $params['CDMARCA'],
                    'CDFORNECED'        => $params['CDFORNECED'],
                    'CDREGIAO'          => $params['CDREGIAO'],
                    'VRPRECUNIITM'      => $params['VRPRECUNIITM'],
                    'P_IDATUREG'        => $params['IDATUREG'],
                    'P_CDOPERINCL'      => $params['CDOPERINCL'],
                    'P_IDATUFORN'       => $params['IDATUFORN'],
                    'P_NRSEQPRELIST'    => $params['NRSEQPRELIST'],
                    'P_IDPRECALT'       => $params['IDPRECALT'],
                    'P_IDITVALDIFLISTA' => $params['IDITVALDIFLISTA'],
                    'P_NRORG'           => $this->environment->getNrorg()
                ];   
            }
            
            $this->supUtilities->crud($query, $param);
        }
        catch (Exception $e) {
            throw new \Exception('Erro ao inserir cotação!');
        }
    }
    
    public function insertFornecedor($params){    
        try {
            $query = "INSERT INTO PZOPGTITMCOL ( CDFILIAL, NRCOLEPRECO, CDFORNECED, CDPRODUTO, CDMARCA, CDREGIAO, NRDIASPGTCOL, VRPERCPGTCOL, NRORG ) SELECT :P_CDFILIAL, :P_NRCOLEPRECO, :P_CDFORNECED, :P_CDPRODUTO, :P_CDMARCA, :P_CDREGIAO, :P_NRDIASPGTCOL, :P_VRPERCPGTCOL, :P_NRORG FROM DUAL WHERE NOT EXISTS (SELECT 1 FROM PZOPGTITMCOL P WHERE P.CDFILIAL   = :P_CDFILIAL AND P.NRCOLEPRECO  = :P_NRCOLEPRECO AND P.CDFORNECED   = :P_CDFORNECED AND P.CDPRODUTO    = :P_CDPRODUTO AND P.CDMARCA      = :P_CDMARCA AND P.CDREGIAO     = :P_CDREGIAO AND P.NRDIASPGTCOL = :P_NRDIASPGTCOL )";
            $param = [
                "P_CDFILIAL"     => $params['CDFILIAL'],
                "P_NRCOLEPRECO"  => $params['NRCOLEPRECO'],    
                "P_CDPRODUTO"    => $params['CDPRODUTO'],    
                "P_CDMARCA"      => $params['CDMARCA'],    
                "P_CDFORNECED"   => $params['CDFORNECED'],    
                "P_CDREGIAO"     => $params['CDREGIAO'],    
                "P_NRDIASPGTCOL" => $params['NRDIASPGTOFO'],    
                "P_VRPERCPGTCOL" => $params['NRDIASPGTOFO'],    
                'P_NRORG'        => $this->environment->getNrorg()
            ];
            return $this->supUtilities->crud($query, $param);
        }
        catch (Exception $e) {
            throw new \Exception('Erro ao inserir cotação!');
        }
    }
    
    public function insertProdutoFornecedor($params, $paramAux){    
        try {
            $query = "INSERT INTO PZOPGTITMCOL (CDFILIAL, NRCOLEPRECO, CDFORNECED, CDPRODUTO, CDMARCA, CDREGIAO, NRDIASPGTCOL, VRPERCPGTCOL, NRORG) SELECT ITEMCOLE.CDFILIAL, ITEMCOLE.NRCOLEPRECO, ITEMCOLE.CDFORNECED, ITEMCOLE.CDPRODUTO, ITEMCOLE.CDMARCA, ITEMCOLE.CDREGIAO, PZOPGTPRFO.NRDIASPGPRFO, PZOPGTPRFO.VRPERCPGPRFO, PZOPGTPRFO.NRORG FROM ITEMCOLE, PZOPGTPRFO WHERE ITEMCOLE.IDTPFOPGTCOL IS NOT NULL AND ITEMCOLE.CDFILIAL = :P_CDFILIAL AND ITEMCOLE.NRCOLEPRECO = :P_NRCOLEPRECO AND ITEMCOLE.CDFORNECED = :P_CDFORNECED AND ITEMCOLE.CDPRODUTO = :P_CDPRODUTO AND ITEMCOLE.CDMARCA = :P_CDMARCA AND ITEMCOLE.CDREGIAO = :P_CDREGIAO AND ITEMCOLE.NRORG = PZOPGTPRFO.NRORG AND PZOPGTPRFO.NRORG = :P_NRORG AND PZOPGTPRFO.CDPRODUTO = :P_PRODFORN AND PZOPGTPRFO.CDFORNECED = ITEMCOLE.CDFORNECED";
            $param = [
                "P_CDFILIAL"    => $params['CDFILIAL'],
                "P_NRCOLEPRECO" => $params['NRCOLEPRECO'],    
                "P_CDPRODUTO"   => $params['CDPRODUTO'],    
                "P_CDMARCA"     => $params['CDMARCA'],    
                "P_CDFORNECED"  => $params['CDFORNECED'],    
                "P_CDREGIAO"    => $params['CDREGIAO'],    
                "P_PRODFORN"    => $params['CDREGIAO'],    
                'P_NRORG'       => $this->environment->getNrorg()
            ];
            return $this->supUtilities->crud($query, $param);
        }
        catch (Exception $e) {
            throw new \Exception('Erro ao inserir cotação!');
        }
    }
    
    public function insertItemCotacaoReajuste($params){    
        try {
            $query = "INSERT INTO MOVOPITEMCOT (CDFILIALMCOT, NRCOLEPRECOMCOT, CDMARCAMCOT, CDREGIAOMCOT, CDFORNECEDMCOT, CDPRODUTOMCOT, DTOPERAITMCOT, CDOPERADORMCOT, DSALTERITMCOT, DSJUSTITMCOT, DSRDMUSUMCOT, DSENDUSUMCOT, DSNMUSUMCOT, DSMETUSUMCOT) SELECT CDFILIAL, NRCOLEPRECO, CDMARCA, CDREGIAO, CDFORNECED, CDPRODUTO, SYSDATE, :P_CDOPERADOR, :P_DESCRICAO || TO_CHAR(ITEMCOLE.VRPRECUNIITM) ||' / '||TO_CHAR(ITEMCOLE.VRPRECUNIITM+(ITEMCOLE.VRPRECUNIITM*(:P_PORCENT / 100))), :P_DESCRICAO, :P_DSRDMUSUMCOT, :P_DSENDUSUMCOT, :P_DSNMUSUMCOT, :P_DSMETUSUMCOT FROM ITEMCOLE WHERE ITEMCOLE.CDFILIAL = :P_CDFILCENT AND ITEMCOLE.NRCOLEPRECO = :P_NRCOLEPRECO AND ITEMCOLE.NRORG = :P_NRORG AND EXISTS(SELECT PRODUTO.CDPRODUTO FROM PRODUTO WHERE ITEMCOLE.CDPRODUTO = PRODUTO.CDPRODUTO AND PRODUTO.NRORG = :P_NRORG AND (PRODUTO.CDARVPROD BETWEEN :P_CDINI AND :P_CDFIN OR :P_TODOS = 'T'))";
            $param = [
                "P_CDOPERADOR"   => $params['CDOPERINCL'],
                "P_DESCRICAO"    => $params['DESCRICAO'],
                "P_PORCENT"      => $params['REAJUSTE'],
                "P_DSRDMUSUMCOT" => $params['DSRDMUSUMCOT'],
                "P_DSENDUSUMCOT" => $params['DSENDUSUMCOT'],
                "P_DSNMUSUMCOT"  => $params['DSNMUSUMCOT'],
                "P_DSMETUSUMCOT" => $params['DSMETUSUMCOT'],
                "P_CDFILCENT"    => $params['CDFILCENT'],
                "P_NRCOLEPRECO"  => $params['NRCOLEPRECO'],
                "P_CDINI"        => $params['CDARVPRODINI'],
                "P_CDFIN"        => $params['CDARVPRODFIN'],
                "P_TODOS"        => $params['TODOSPROD'],
                'P_NRORG'        => $this->environment->getNrorg()
            ];
            return $this->supUtilities->crud($query, $param);
        }
        catch (Exception $e) {
            throw new \Exception('Erro ao inserir cotação!');
        }
    }
    
    public function insertParcelas($params){    
        try {
            $query = "INSERT INTO PZOPGTITMCOL (CDFILIAL, NRCOLEPRECO, CDFORNECED, CDPRODUTO, CDMARCA, CDREGIAO, NRDIASPGTCOL, VRPERCPGTCOL, NRORG) VALUES (:CDFILIAL, :NRCOLEPRECO, :CDFORNECED, :CDPRODUTO, :CDMARCA, :CDREGIAO, :NRDIASPGTCOL, :VRPERCPGTCOL, :NRORG)";
            $param = [
                "CDFILIAL"      => $params['CDFILIAL'],
                "NRCOLEPRECO"   => $params['NRCOLEPRECO'],
                "CDFORNECED"    => $params['CDFORNECED'],
                "CDPRODUTO"     => $params['CDPRODUTO'],
                "CDMARCA"       => $params['CDMARCA'],
                "CDREGIAO"      => $params['CDREGIAO'],
                "NRDIASPGTCOL"  => $params['NRDIASPGTOFO'],
                "VRPERCPGTCOL"  => $params['VRPERCPGTOFO'],
                'NRORG'         => $this->environment->getNrorg()
            ];
            return $this->supUtilities->crud($query, $param);
        }
        catch (Exception $e) {
            throw new \Exception('Erro ao inserir cotação!');
        }
    }
    
    public function insertJustificativa($params){    
        try {
            $query = "INSERT INTO MOVOPITEMCOT (CDFILIALMCOT, NRCOLEPRECOMCOT, CDMARCAMCOT, CDREGIAOMCOT, CDFORNECEDMCOT, CDPRODUTOMCOT, DTOPERAITMCOT, CDOPERADORMCOT, DSALTERITMCOT, DSJUSTITMCOT, DSRDMUSUMCOT, DSENDUSUMCOT, DSNMUSUMCOT, DSMETUSUMCOT, NRORG) VALUES (:CDFILIALMCOT, :NRCOLEPRECOMCOT, :CDMARCAMCOT, :CDREGIAOMCOT, :CDFORNECEDMCOT, :CDPRODUTOMCOT, SYSDATE, :CDOPERADORMCOT, :DSALTERITMCOT, :DSJUSTITMCOT, :DSRDMUSUMCOT, :DSENDUSUMCOT, :DSNMUSUMCOT, :DSMETUSUMCOT, :NRORG)";
            $param = [
                "CDFILIALMCOT"     => $params['CDFILIAL'],
                "NRCOLEPRECOMCOT"  => $params['NRCOLEPRECO'],
                "CDFORNECEDMCOT"   => $params['CDFORNECED'],
                "CDPRODUTOMCOT"    => $params['CDPRODUTO'],
                "CDMARCAMCOT"      => $params['CDMARCA'],
                "CDREGIAOMCOT"     => $params['CDREGIAO'],
                "CDOPERADORMCOT"   => $params['CDOPERADOR'],
                "DSALTERITMCOT"    => $params['DSALTERITM'],
                "DSJUSTITMCOT"     => $params['DSJUSTITM'],
                "DSRDMUSUMCOT"     => null,
                "DSENDUSUMCOT"     => null,
                "DSNMUSUMCOT"      => null,
                "DSMETUSUMCOT"     => null,
                'NRORG'            => $this->environment->getNrorg()
            ];
            return $this->supUtilities->crud($query, $param);
        }
        catch (Exception $e) {
            throw new \Exception('Erro ao inserir cotação!');
        }
    }
    
    
    /********************
     *      UPDATES     *
     * ******************/
     
    private function getIndiceValue($row){    
        $query = "SELECT VRINDICE 
                        FROM MOVINDICE
                       WHERE CDTPINDICE = :P_CDTPINDICE
                         AND IDPERVALOR = :P_IDPERVALOR
                         AND CDINDICE   = :P_CDINDICE
                         AND DTREFIND   = :P_DTREFINDCOL
                         AND NRORG      = :P_NRORG";
        $params = [
            'P_CDTPINDICE' => $row['CDTPINDICE'],
            'P_IDPERVALOR' => $row['IDPERVALOR'],
            'P_CDINDICE'   => $row['CDINDICE'],
            'P_DTREFINDCOL'=> $row['DTREFIND'],
            'P_NRORG'      => $this->environment->getNrorg()
        ];
        $result = $this->supUtilities->crud($query, $params);
        if($result){
            if($row['IDPERVALOR'] == '2'){
                $vrIndice = $result[0]['VRINDICE'];
            } else {
                $vrIndice = $result[0]['VRINDICE']/100;
            }
        }
        
        return $vrIndice;
    }
    
    public function salvaCotacaoEditada($row){  
        try {
            if(isset($row['CDTPINDICE'])){
                $query = "UPDATE COLETAPR SET CDOPERADOR = :CDOPERADOR, IDTIPCOLREG = :IDTIPCOLREG, IDDISMONTLIS = :IDDISMONTLIS, DSOBSCOTPREC = :DSOBSCOTPREC, IDSITUCOLETA = :IDSITUCOLETA, DTCONCOLEPREC = :P_DTCONCOLEPREC, CDTPINDICECOL = :P_CDTPINDICECOL, IDPERVALORCOL = :P_IDPERVALORCOL, CDINDICECOL = :P_CDINDICECOL, DTREFINDCOL = :P_DTREFINDCOL WHERE CDFILIAL = :OLD_CDFILIAL AND NRCOLEPRECO = :OLD_NRCOLEPRECO";
                $params = [
                    'CDOPERADOR'      => $row['CDOPERADOR'],
                    'IDTIPCOLREG'     => $row['IDTIPCOLREG'],
                    'IDDISMONTLIS'    => $row['IDDISMONTLIS'],
                    'DSOBSCOTPREC'    => $row['DSOBSCOTPREC'],
                    'IDSITUCOLETA'    => $row['IDSITUCOLETA'],
                    'P_DTCONCOLEPREC' => $row['DTCONCOLEPREC'],
                    'OLD_CDFILIAL'    => $row['CDFILIAL'],
                    'OLD_NRCOLEPRECO' => $row['NRCOLEPRECO'],
                    'P_CDTPINDICECOL' => $row['CDTPINDICE'],
                    'P_IDPERVALORCOL' => $row['IDPERVALOR'],
                    'P_CDINDICECOL'   => $row['CDINDICE'],
                    'P_DTREFINDCOL'   => $row['DTREFIND']
                ];
                $this->supUtilities->crud($query, $params);
                
                //ITEMCOLE
                $query = "UPDATE ITEMCOLE SET VRINDICEITMCOL = (VRPRECUNIITM / :P_INDICE) WHERE NVL(VRPRECUNIITM, 0) > 0 AND CDFILIAL = :P_CDFILIAL AND NRCOLEPRECO = :P_NRCOLEPRECO AND NRORG = :NRORG";
                $params = [
                    'P_CDFILIAL'    => $row['CDFILIAL'],
                    'P_NRCOLEPRECO' => $row['NRCOLEPRECO'],
                    'P_INDICE'      => $this->getIndiceValue($row),
                    'NRORG'         => $this->environment->getNrorg()
                ];
                $this->supUtilities->crud($query, $params);
                
            } else {
                $query = "UPDATE COLETAPR SET CDOPERADOR  = :CDOPERADOR, IDTIPCOLREG   = :IDTIPCOLREG, IDDISMONTLIS  = :IDDISMONTLIS, DSOBSCOTPREC  = :DSOBSCOTPREC, IDSITUCOLETA  = :IDSITUCOLETA, DTCONCOLEPREC = :P_DTCONCOLEPREC WHERE CDFILIAL  = :OLD_CDFILIAL AND NRCOLEPRECO = :OLD_NRCOLEPRECO";
                $params = [
                    'CDOPERADOR'      => $row['CDOPERADOR'],
                    'IDTIPCOLREG'     => $row['IDTIPCOLREG'],
                    'IDDISMONTLIS'    => $row['IDDISMONTLIS'],
                    'DSOBSCOTPREC'    => $row['DSOBSCOTPREC'],
                    'IDSITUCOLETA'    => $row['IDSITUCOLETA'],
                    'P_DTCONCOLEPREC' => $row['DTCONCOLEPREC'],
                    'OLD_CDFILIAL'    => $row['CDFILIAL'],
                    'OLD_NRCOLEPRECO' => $row['NRCOLEPRECO']
                ];
                $this->supUtilities->crud($query, $params);
            }
        }
        catch (Exception $e) {
            throw new \Exception('Erro ao atualizar os registros!');
        }
    }
    
    public function updateRegForn($row){    
        try {
            $query = "UPDATE ITEMCOLE SET VRPRECUNIITM = :P_VRPRECUNIITM, IDITMCOTPREC = 'S', IDATUREGIALT = :P_IDATUREG, IDATUFORNALT = :P_IDATUFORN WHERE CDFILIAL = :P_CDFILIAL AND NRCOLEPRECO = :P_NRCOLEPRECO AND CDREGIAO = :P_CDREGIAO AND CDFORNECED = :P_CDFORNECED AND CDPRODUTO = :P_CDPRODUTO AND CDMARCA = :P_CDMARCA AND NRORG = :P_NRORG AND NOT EXISTS (SELECT CDFORNECED, CDREGIAO FROM REFOREG WHERE REFOREG.CDFORNECED = ITEMCOLE.CDFORNECED AND REFOREG.CDREGIAO = ITEMCOLE.CDREGIAO)";
            $params = [
                'P_VRPRECUNIITM' => $row['VRPRECUNIITM'],
                'P_IDATUREG'     => $row['P_IDATUREG'],
                'P_IDATUFORN'    => $row['P_IDATUFORN'],
                'P_CDFILIAL'     => $row['CDFILIAL'],
                'P_NRCOLEPRECO'  => $row['NRCOLEPRECO'],
                'P_CDREGIAO'     => $row['CDREGIAO'],
                'P_CDFORNECED'   => $row['CDFORNECED'],
                'P_CDPRODUTO'    => $row['CDPRODUTO'],
                'P_CDMARCA'      => $row['CDMARCA'],
                'P_NRORG'        => $this->environment->getNrorg()
            ];
            $this->supUtilities->crud($query, $params);
        }
        catch (Exception $e) {
            throw new \Exception('Erro ao atualizar os registros!');
        }
    }
    
    public function updRegFornIndice($row){    
        try {
            $qryIndice = "SELECT VRINDICE 
                            FROM MOVINDICE
                           WHERE CDTPINDICE = :P_CDTPINDICE
                             AND IDPERVALOR = :P_IDPERVALOR
                             AND CDINDICE = :P_CDINDICE
                             AND DTREFIND = :P_DTREFINDCOL
                             AND NRORG = :P_NRORG";
            $parIndice = [
                'P_CDTPINDICE' => $row['CDTPINDICECOL'],
                'P_IDPERVALOR' => $row['IDPERVALORCOL'],
                'P_CDINDICE'   => $row['CDINDICECOL'],
                'P_DTREFINDCOL'=> $row['DTREFINDCOL'],
                'P_NRORG'      => $this->environment->getNrorg()
            ];
            $vrIndice = $this->supUtilities->crud($qryIndice, $parIndice);
            if($vrIndice){
                if($row['IDPERVALORCOL'] == '2'){
                    $row['VRPRECUNIITM'] = $row['VRINDICEITMCOL'] * $vrIndice[0]['VRINDICE'];
                } else {
                    $row['VRPRECUNIITM'] = $row['VRINDICEITMCOL'] * ($vrIndice[0]['VRINDICE']/100);
                }
            }
            
            $query = "UPDATE ITEMCOLE SET VRPRECUNIITM = :P_VRPRECUNIITM, VRINDICEITMCOL = :P_VRINDICEITMCOL, IDITMCOTPREC = 'S', IDATUREGIALT = :P_IDATUREG, IDATUFORNALT = :P_IDATUFORN WHERE CDFILIAL = :P_CDFILIAL AND NRCOLEPRECO = :P_NRCOLEPRECO AND CDREGIAO = :P_CDREGIAO AND CDFORNECED = :P_CDFORNECED AND CDPRODUTO = :P_CDPRODUTO AND CDMARCA = :P_CDMARCA AND NRORG = :P_NRORG AND NOT EXISTS (SELECT CDFORNECED, CDREGIAO FROM REFOREG WHERE REFOREG.CDFORNECED = ITEMCOLE.CDFORNECED AND REFOREG.CDREGIAO = ITEMCOLE.CDREGIAO)";
            $params = [
                'P_VRPRECUNIITM'    => $row['VRPRECUNIITM'],
                'P_VRINDICEITMCOL'  => $row['VRINDICEITMCOL'],
                'P_IDATUREG'        => $row['P_IDATUREG'],
                'P_IDATUFORN'       => $row['P_IDATUFORN'],
                'P_CDFILIAL'        => $row['CDFILIAL'],
                'P_NRCOLEPRECO'     => $row['NRCOLEPRECO'],
                'P_CDREGIAO'        => $row['CDREGIAO'],
                'P_CDFORNECED'      => $row['CDFORNECED'],
                'P_CDPRODUTO'       => $row['CDPRODUTO'],
                'P_CDMARCA'         => $row['CDMARCA'],
                'P_NRORG'           => $this->environment->getNrorg()
            ];
            $this->supUtilities->crud($query, $params);
        }
        catch (Exception $e) {
            throw new \Exception('Erro ao atualizar os registros!');
        }
    }
    
    public function upPrazoPagamentotCol($params, $paramAux){
        try {
            $query = "UPDATE ITEMCOLE SET CDFORMPGTO = :P_CDFORMPGTO, IDTPFOPGTCOL = :P_IDTPFOPGTCOL, NRDIAPGFXCOL = :P_NRDIAPGFXCOL, NRPARCPGTCOL = :P_NRPARCPGTCOL, IDUTDIAPGCOL = :P_IDUTDIAPGCOL WHERE CDFILIAL = :P_CDFILIAL AND NRCOLEPRECO = :P_NRCOLEPRECO AND CDPRODUTO = :P_CDPRODUTO AND CDMARCA = :P_CDMARCA AND CDFORNECED = :P_CDFORNECED AND CDREGIAO = :P_CDREGIAO";
            $params = [
                "P_CDFILIAL"     => $params['CDFILIAL'],
                "P_NRCOLEPRECO"  => $params['NRCOLEPRECO'],
                "P_CDPRODUTO"    => $params['CDPRODUTO'],
                "P_CDMARCA"      => $params['CDMARCA'],
                "P_CDFORNECED"   => $params['CDFORNECED'],
                "P_CDREGIAO"     => $params['CDREGIAO'],
                "P_CDFORMPGTO"   => $paramAux['CDFORMPGTO'],
                "P_IDTPFOPGTCOL" => $paramAux['IDTPFOPGTCOL'],
                "P_NRDIAPGFXCOL" => $paramAux['NRDIAPGFXCOL'],
                "P_NRPARCPGTCOL" => $paramAux['NRPARCPGTCOL'],
                "P_IDUTDIAPGCOL" => $paramAux['IDUTDIAPGCOL']
            ];
            $this->supUtilities->crud($query, $params);
        }
        catch (Exception $e) {
            throw new \Exception('Erro ao atualizar os registros!');
        }
    }
    
    public function forcTrigger($params){    
        try {
            $query = "UPDATE ITEMCOLE SET IDTPFOPGTCOL = IDTPFOPGTCOL WHERE CDFILIAL = :P_CDFILIAL AND NRCOLEPRECO = :P_NRCOLEPRECO AND CDPRODUTO = :P_CDPRODUTO AND CDMARCA = :P_CDMARCA AND CDFORNECED = :P_CDFORNECED AND CDREGIAO = :P_CDREGIAO";
            $params = [
                "P_CDFILIAL"    => $params['CDFILIAL'],
                "P_NRCOLEPRECO" => $params['NRCOLEPRECO'],    
                "P_CDPRODUTO"   => $params['CDPRODUTO'],    
                "P_CDMARCA"     => $params['CDMARCA'],    
                "P_CDFORNECED"  => $params['CDFORNECED'],    
                "P_CDREGIAO"    => $params['CDREGIAO']    
            ];
            $this->supUtilities->crud($query, $params);
        }
        catch (Exception $e) {
            throw new \Exception('Erro ao atualizar os registros!');
        }
    }
    
    public function updateValidadeColeta($params){    
        try {
            $query = "UPDATE COLETAPR SET DTINIVALCOL = :P_DTINIVALCOL, DTFINVALCOL = :P_DTFINVALCOL WHERE CDFILIAL = :P_CDFILIAL AND NRCOLEPRECO = :P_NRCOLEPRECO AND NRORG = :P_NRORG";
            $params = [
                "P_CDFILIAL"    => $params['CDFILIAL'],
                "P_NRCOLEPRECO" => $params['NRCOLEPRECO'],    
                "P_DTINIVALCOL" => $params['DTINIVALITM'],    
                "P_DTFINVALCOL" => $params['DTFINVALITM'],  
                'P_NRORG'       => $this->environment->getNrorg() 
            ];
            $this->supUtilities->crud($query, $params);
        }
        catch (Exception $e) {
            throw new \Exception('Erro ao atualizar os registros!');
        }
    }
    
    public function updateValidadeItemColeta($params){    
        try {
            $query = "UPDATE ITEMCOLE SET DTINIVALITM = :P_DTINIVALITM, DTFINVALITM = :P_DTFINVALITM, IDITMLISPREC = 'N', NRSEQPRELIST = NULL, IDPRECALT = 'N' WHERE CDFILIAL = :P_CDFILIAL AND NRCOLEPRECO = :P_NRCOLEPRECO AND NRORG = :P_NRORG";
            $params = [
                "P_CDFILIAL"    => $params['CDFILIAL'],
                "P_NRCOLEPRECO" => $params['NRCOLEPRECO'],    
                "P_DTINIVALITM" => $params['DTINIVALITM'],    
                "P_DTFINVALITM" => $params['DTFINVALITM'],  
                'P_NRORG'       => $this->environment->getNrorg() 
            ];
            $this->supUtilities->crud($query, $params);
        }
        catch (Exception $e) {
            throw new \Exception('Erro ao atualizar os registros!');
        }
    }
    
    public function updateDataPreco($params){    
        try {
            $query = "UPDATE ITEMCOLE SET ITEMCOLE.VRPRECUNIITM = ITEMCOLE.VRPRECUNIITM+(ITEMCOLE.VRPRECUNIITM*(:P_PORCENT / 100)) WHERE ITEMCOLE.CDFILIAL = :P_CDFILCENT AND ITEMCOLE.NRCOLEPRECO = :P_NRCOLEPRECO AND ITEMCOLE.NRORG = :P_NRORG AND EXISTS(SELECT PRODUTO.CDPRODUTO FROM PRODUTO WHERE ITEMCOLE.CDPRODUTO = PRODUTO.CDPRODUTO AND ITEMCOLE.NRORG = PRODUTO.NRORG AND (PRODUTO.CDARVPROD BETWEEN :P_CDINI AND :P_CDFIN OR :P_TODOS = 'T'))";
            $params = [
                "P_PORCENT"     => $params['REAJUSTE'],
                "P_CDFILCENT"   => $params['CDFILCENT'],    
                "P_NRCOLEPRECO" => $params['NRCOLEPRECO'],    
                "P_CDINI"       => $params['CDARVPRODINI'],
                "P_CDFIN"       => self::strPadRigthProd($params['CDARVPRODFIN']),
                "P_TODOS"       => $params['TODOSPROD'],
                'P_NRORG'       => $this->environment->getNrorg() 
            ];
            $this->supUtilities->crud($query, $params);
        }
        catch (Exception $e) {
            throw new \Exception('Erro ao atualizar os registros!');
        }
    }
    
    public function updateFreteForn($params){    
        try {
            $query = "UPDATE ITEMCOLE SET VRFRETEFORNEPLANCOT = :P_VRFRETEFORN WHERE CDFILIAL = :P_CDFILIAL AND NRCOLEPRECO = :P_NRCOLEPRECO AND CDFORNECED = :P_CDFORNECED AND NRORG = :P_NRORG";
            $params = [
                "P_VRFRETEFORN" => $params['VRFRETEFORN'],
                "P_CDFILIAL"    => $params['CDFILIAL'],
                "P_NRCOLEPRECO" => $params['NRCOLEPRECO'],    
                "P_CDFORNECED"  => $params['CDFORNECED'],
                'P_NRORG'       => $this->environment->getNrorg() 
            ];
            $this->supUtilities->crud($query, $params);
        }
        catch (Exception $e) {
            throw new \Exception('Erro ao atualizar os registros!');
        }
    }
    
    public function updateItemColePersonalizado($query, $params){    
        try {
            $params = [
                "VRPERDESITM"    => $params['VRPERDESITM'],
                "OLD_CDFILIAL"   => $params['CDFILIAL'],
                "OLD_NRCOLEPRECO"=> $params['NRCOLEPRECO'],
                "P_CDPRODUTO"    => $params['CDPRODUTOPES'],
                "P_CDMARCA"      => $params['CDMARCAPES'],
                "P_CDREGIAO"     => $params['CDREGIAOPES'],
                "P_CDFORNECED"   => $params['CDFORNECEDPES']
            ];
            $this->supUtilities->crud($query, $params);
        }
        catch (Exception $e) {
            throw new \Exception('Erro ao atualizar os registros!');
        }
    }
    
    public function updateItemCole($params){    
        try {
            $query = "UPDATE ITEMCOLE SET CDFILIAL = :CDFILIAL, NRCOLEPRECO = :NRCOLEPRECO, CDFORNECED = :CDFORNECED, CDREGIAO = :CDREGIAO, CDPRODUTO = :CDPRODUTO, CDMARCA = :CDMARCA, CDTIPCOB = :CDTIPCOB, CDSEGMARCCOL = :CDSEGMARCCOL, DTINIVALITM = :DTINIVALITM, DTFINVALITM = :DTFINVALITM, VRPRECUNIITM = :VRPRECUNIITM, VRPRECECOITM = :VRPRECECOITM, IDTIPFREITM = :IDTIPFREITM, NRPZOENTCOL = :NRPZOENTCOL, CDFORMPGTO = :CDFORMPGTO, IDTPFOPGTCOL = :IDTPFOPGTCOL, NRPARCPGTCOL = :NRPARCPGTCOL, NRDIAPGFXCOL = :NRDIAPGFXCOL, IDUTDIAPGCOL = :IDUTDIAPGCOL, QTACMPITMCOL = :QTACMPITMCOL WHERE CDFILIAL = :OLD_CDFILIAL AND NRCOLEPRECO = :OLD_NRCOLEPRECO AND CDFORNECED = :OLD_CDFORNECED AND CDREGIAO = :OLD_CDREGIAO AND CDPRODUTO = :OLD_CDPRODUTO AND CDMARCA = :OLD_CDMARCA AND NRORG = :P_NRORG";
            $params = [
                "CDFILIAL"        => $params['CDFILIAL'],
                "NRCOLEPRECO"     => $params['NRCOLEPRECO'],
                "CDFORNECED"      => $params['CDFORNECED'],
                "CDREGIAO"        => $params['CDREGIAO'],
                "CDPRODUTO"       => $params['CDPRODUTO'],
                "CDMARCA"         => $params['CDMARCA'],
                "CDTIPCOB"        => $params['CDTIPCOB'],
                "CDSEGMARCCOL"    => $params['CDSEGMARCCOL'],
                "DTINIVALITM"     => $params['DTINIVALITM'], 
                "DTFINVALITM"     => $params['DTFINVALITM'], 
                "VRPRECUNIITM"    => $params['VRPRECUNIITM'],
                "VRPRECECOITM"    => $params['VRPRECECOITM'],
                "IDTIPFREITM"     => $params['IDTIPFREITM'],
                "NRPZOENTCOL"     => $params['NRPZOENTCOL'],
                "CDFORMPGTO"      => $params['CDFORMPGTO'],
                "IDTPFOPGTCOL"    => $params['IDTPFORPGTFO'],
                "NRPARCPGTCOL"    => $params['NRPARCPGTOFO'],
                "NRDIAPGFXCOL"    => $params['NRDIAPGFIXFO'],
                "IDUTDIAPGCOL"    => $params['IDUTDIAPGTFO'],
                "QTACMPITMCOL"    => $params['QTACMPITMCOL'],
                "OLD_CDFILIAL"    => $params['CDFILIAL'],
                "OLD_NRCOLEPRECO" => $params['NRCOLEPRECO'],
                "OLD_CDFORNECED"  => $params['CDFORNECED'],
                "OLD_CDREGIAO"    => $params['CDREGIAO'],
                "OLD_CDPRODUTO"   => $params['CDPRODUTO'],
                "OLD_CDMARCA"     => $params['CDMARCA'],
                'P_NRORG'         => $this->environment->getNrorg() 
            ];
            $this->supUtilities->crud($query, $params);
        }
        catch (Exception $e) {
            throw new \Exception('Erro ao atualizar os registros!');
        }
    }
    
    public function carregarAComprar($params){    
        try {
            $query = "UPDATE ITEMCOLE SET QTACMPITMCOL = (SELECT SUM((COMP.QTDECOMP-NVL(JACOMP.QTDECOMP,0)) / PRODUTO.VRFATOCONV) AS QTACOMPRAR FROM PRODUTO, PARASUPRI, (SELECT FILIAL.CDREGIAO, ITEMSOLI.DTPROGENTR, PRODUTO.CDPRODESTO, SUM(ITEMSOLI.QTSOLIPROD*PRODUTO.VRFATOCONV) AS QTDECOMP FROM PRODUTO, ITEMSOLI, FILIAL WHERE (ITEMSOLI.IDITMCONS = 'N') AND (ITEMSOLI.CDFILSOLI = FILIAL.CDFILIAL) AND (ITEMSOLI.CDPRODUTO = PRODUTO.CDPRODUTO) AND (NVL(ITEMSOLI.IDSOLIAPROV,'S') = 'S') GROUP BY FILIAL.CDREGIAO, ITEMSOLI.DTPROGENTR, PRODUTO.CDPRODESTO) COMP, (SELECT FILIAL.CDREGIAO, ITEMAUTF.DTENTRITMSOL, PRODUTO.CDPRODESTO, SUM(ITEMAUTF.QTPROGENTAF*PRODUTO.VRFATOCONV) AS QTDECOMP FROM PRODUTO, ITEMAUTF, AUTOFORNE, FILIAL WHERE (AUTOFORNE.CDFILDEST = ITEMAUTF.CDFILDEST) AND (AUTOFORNE.CDFILLANC = ITEMAUTF.CDFILLANC) AND (AUTOFORNE.NRAUTOFORN = ITEMAUTF.NRAUTOFORN) AND (ITEMAUTF.CDFILDEST = FILIAL.CDFILIAL) AND (ITEMAUTF.CDPRODUTO = PRODUTO.CDPRODUTO) AND (AUTOFORNE.IDSITUATUAAF IS NULL) AND (AUTOFORNE.IDAFFORALIM = 'N') AND (AUTOFORNE.IDAUTOFORNEX = 'N') AND (NVL(AUTOFORNE.IDBAIXAAF, '0') <> '5') GROUP BY FILIAL.CDREGIAO, ITEMAUTF.DTENTRITMSOL, PRODUTO.CDPRODESTO) JACOMP WHERE COMP.CDREGIAO = ITEMCOLE.CDREGIAO AND COMP.DTPROGENTR BETWEEN ITEMCOLE.DTINIVALITM AND ITEMCOLE.DTFINVALITM AND PRODUTO.CDPRODUTO = ITEMCOLE.CDPRODUTO AND PRODUTO.CDPRODESTO = COMP.CDPRODESTO AND COMP.CDREGIAO = JACOMP.CDREGIAO (+) AND COMP.DTPROGENTR = JACOMP.DTENTRITMSOL (+) AND COMP.CDPRODESTO = JACOMP.CDPRODESTO (+) AND COMP.QTDECOMP-NVL(JACOMP.QTDECOMP,0) > 0 AND PARASUPRI.CDFILIAL = :P_CDFILIAL AND NVL(PARASUPRI.IDCONSPRSOL, 'N') = 'N' HAVING SUM((COMP.QTDECOMP-NVL(JACOMP.QTDECOMP,0)) / PRODUTO.VRFATOCONV) <> 0 UNION ALL SELECT SUM(COMP.QTDECOMP-NVL(JACOMP.QTDECOMP,0)) AS QTACOMPRAR FROM PARASUPRI, (SELECT FILIAL.CDREGIAO, ITEMSOLI.DTPROGENTR, ITEMSOLI.CDPRODUTO, SUM(ITEMSOLI.QTSOLIPROD) AS QTDECOMP FROM ITEMSOLI, FILIAL WHERE (ITEMSOLI.IDITMCONS = 'N') AND (ITEMSOLI.CDFILSOLI = FILIAL.CDFILIAL) AND (NVL(ITEMSOLI.IDSOLIAPROV,'S') = 'S') GROUP BY FILIAL.CDREGIAO, ITEMSOLI.DTPROGENTR, ITEMSOLI.CDPRODUTO) COMP, (SELECT FILIAL.CDREGIAO, ITEMAUTF.DTENTRITMSOL, ITEMAUTF.CDPRODUTO, SUM(ITEMAUTF.QTPROGENTAF) AS QTDECOMP FROM ITEMAUTF, AUTOFORNE, FILIAL WHERE (AUTOFORNE.CDFILDEST = ITEMAUTF.CDFILDEST) AND (AUTOFORNE.CDFILLANC = ITEMAUTF.CDFILLANC) AND (AUTOFORNE.NRAUTOFORN = ITEMAUTF.NRAUTOFORN) AND (ITEMAUTF.CDFILDEST = FILIAL.CDFILIAL) AND (AUTOFORNE.IDSITUATUAAF IS NULL) AND (AUTOFORNE.IDAFFORALIM = 'N') AND (AUTOFORNE.IDAUTOFORNEX = 'N') AND (NVL(AUTOFORNE.IDBAIXAAF, '0') <> '5') GROUP BY FILIAL.CDREGIAO, ITEMAUTF.DTENTRITMSOL, ITEMAUTF.CDPRODUTO) JACOMP WHERE COMP.CDREGIAO = ITEMCOLE.CDREGIAO AND COMP.DTPROGENTR BETWEEN ITEMCOLE.DTINIVALITM AND ITEMCOLE.DTFINVALITM AND COMP.CDPRODUTO = ITEMCOLE.CDPRODUTO AND COMP.CDREGIAO = JACOMP.CDREGIAO (+) AND COMP.DTPROGENTR = JACOMP.DTENTRITMSOL (+) AND COMP.CDPRODUTO = JACOMP.CDPRODUTO (+) AND COMP.QTDECOMP-NVL(JACOMP.QTDECOMP,0) > 0 AND PARASUPRI.CDFILIAL = :P_CDFILIAL AND PARASUPRI.IDCONSPRSOL = 'S' HAVING SUM(COMP.QTDECOMP-NVL(JACOMP.QTDECOMP,0)) <> 0) WHERE CDFILIAL = :P_CDFILIAL AND NRORG = :P_NRORG AND NRCOLEPRECO = :P_NRCOLEPRECO";
            $params = [
                "P_CDFILIAL"    => $params['CDFILIAL'],
                "P_NRCOLEPRECO" => $params['NRCOLEPRECO'],
                'P_NRORG'       => $this->environment->getNrorg() 
            ];
            $this->supUtilities->crud($query, $params);
        }
        catch (Exception $e) {
            throw new \Exception('Erro ao atualizar os registros!');
        }
    }
    
    public function updateItem($params){    
        try {
            $query = "UPDATE ITEMCOLE SET CDFORMPGTO = NULL, IDTPFOPGTCOL = NULL, NRDIAPGFXCOL = NULL, NRPARCPGTCOL = NULL, IDUTDIAPGCOL = NULL WHERE CDFILIAL = :P_CDFILIAL AND NRCOLEPRECO = :P_NRCOLEPRECO AND NRORG = :P_NRORG";
            $params = [
                "P_CDFILIAL"    => $params['CDFILIAL'],
                "P_NRCOLEPRECO" => $params['NRCOLEPRECO'],
                'P_NRORG'       => $this->environment->getNrorg() 
            ];
            $this->supUtilities->crud($query, $params);
        }
        catch (Exception $e) {
            throw new \Exception('Erro ao atualizar os registros!');
        }
    }
    
    public function updatePreEcZerado($params){    
        try {
            $query = "UPDATE ITEMCOLE SET VRPRECUNIITM = VRPRECUNIITM WHERE CDFILIAL = :P_CDFILIAL AND NRCOLEPRECO = :P_NRCOLEPRECO AND NRORG = :P_NRORG AND VRPRECUNIITM > 0 AND VRPRECECOITM <= 0";
            $params = [
                "P_CDFILIAL"    => $params['CDFILIAL'],
                "P_NRCOLEPRECO" => $params['NRCOLEPRECO'],
                'P_NRORG'       => $this->environment->getNrorg() 
            ];
            $this->supUtilities->crud($query, $params);
        }
        catch (Exception $e) {
            throw new \Exception('Erro ao atualizar os registros!');
        }
    }
    
    public function salvarObservacao($params){    
        try {
            $query = "UPDATE ITEMCOLE SET DSOBSITMCOL = :P_DSOBSITMCOL WHERE CDFILIAL = :P_CDFILIAL AND NRCOLEPRECO = :P_NRCOLEPRECO AND CDREGIAO = :P_CDREGIAO AND CDFORNECED = :P_CDFORNECED AND CDPRODUTO = :P_CDPRODUTO AND CDMARCA = :P_CDMARCA AND NRORG = :P_NRORG";
            $params = [
                "P_DSOBSITMCOL"   => $params['DSOBSITMCOL'],
                "P_CDFILIAL"      => $params['CDFILIAL'],
                "P_NRCOLEPRECO"   => $params['NRCOLEPRECO'],
                "P_CDREGIAO"      => $params['CDREGIAO'],
                "P_CDFORNECED"    => $params['CDFORNECED'],
                "P_CDPRODUTO"     => $params['CDPRODUTO'],
                "P_CDMARCA"       => $params['CDMARCA'],
                'P_NRORG'         => $this->environment->getNrorg()
            ];
            $this->supUtilities->crud($query, $params);
        }
        catch (Exception $e) {
            throw new \Exception('Erro ao atualizar os registros!');
        }
    }
    
    public function updateParametrosCotacao($params){    
        try {
            $query = "UPDATE COLETAPR SET IDTIPCOLREG = 'G' WHERE CDFILIAL = :P_CDFILIAL AND NRCOLEPRECO = :P_NRCOLEPRECO AND NVL(IDTIPCOLREG, 'G') NOT IN ('E', 'W')";
            $params = [
                "P_CDFILIAL"      => $params['CDFILIAL'],
                "P_NRCOLEPRECO"   => $params['NRCOLEPRECO'],
            ];
            $this->supUtilities->crud($query, $params);
        }
        catch (Exception $e) {
            throw new \Exception('Erro ao atualizar os registros!');
        }
    }
    
    /********************
     *      DELETES     *
     * ******************/
     
    public function deletaCotacao($row){    
        try {
            $query = "DELETE FROM COLETAPR WHERE CDFILIAL = :OLD_CDFILIAL AND NRCOLEPRECO = :OLD_NRCOLEPRECO AND NRORG = :P_NRORG";
            $params = [
                'OLD_CDFILIAL'    => $row['CDFILIAL'],
                'OLD_NRCOLEPRECO' => $row['NRCOLEPRECO'],
                'P_NRORG'         => $this->environment->getNrorg()
            ];
            $this->supUtilities->crud($query, $params);
        }
        catch (Exception $e) {
            throw new \Exception('Erro ao excluir os registros!');
        }
    }
    
    public function deleteItemPrecoZerado($row){    
        try {
            $query = "DELETE FROM ITEMCOLE WHERE CDFILIAL = :P_CDFILIAL AND NRCOLEPRECO = :P_NRCOLEPRECO AND NRORG = :P_NRORG AND CDREGIAO = NVL(:P_CDREGIAO, CDREGIAO) AND VRPRECUNIITM = 0";
            $params = [
                'P_CDFILIAL'    => $row['CDFILIAL'],
                'P_NRCOLEPRECO' => $row['NRCOLEPRECO'],
                'P_CDREGIAO'    => $row['CDREGIAO'],
                'P_NRORG'       => $this->environment->getNrorg()
            ];
            $this->supUtilities->crud($query, $params);
        }
        catch (Exception $e) {
            throw new \Exception('Erro ao excluir os registros!');
        }
    }
    
    public function deletaRegiao($params){    
        try {
            $query = "DELETE FROM ITEMCOLE WHERE CDFILIAL = :P_CDFILIAL AND NRCOLEPRECO = :P_NRCOLEPRECO AND CDREGIAO = :P_CDREGIAO AND CDFORNECED = NVL(:P_CDFORNECED, CDFORNECED) AND NRORG = :P_NRORG";
            $params = [
                "P_CDFILIAL"    => $params['CDFILIAL'],
                "P_NRCOLEPRECO" => $params['NRCOLEPRECO'],    
                "P_CDREGIAO"    => $params['CDREGIAO'],
                "P_CDFORNECED"  => $params['CDFORNECED'], 
                'P_NRORG'       => $this->environment->getNrorg()
            ];
            $this->supUtilities->crud($query, $params);
        }
        catch (Exception $e) {
            throw new \Exception('Erro ao excluir os registros!');
        }
    }
    
    public function deletaFornecedor($params){
        try {
            $query = "DELETE FROM ITEMCOLE WHERE CDFILIAL = :P_CDFILIAL AND NRCOLEPRECO = :P_NRCOLEPRECO AND CDFORNECED = :P_CDFORNECED AND (CDREGIAO = :P_CDREGIAO OR :P_CDREGIAO = 'T') AND NRORG = :P_NRORG";
            $params = [
                "P_CDFILIAL"    => $params['CDFILIAL'],
                "P_NRCOLEPRECO" => $params['NRCOLEPRECO'],    
                "P_CDREGIAO"    => $params['CDREGIAO'],
                "P_CDFORNECED"  => $params['CDFORNECED'], 
                'P_NRORG'       => $this->environment->getNrorg()
            ];
            $this->supUtilities->crud($query, $params);
        }
        catch (Exception $e) {
            throw new \Exception('Erro ao excluir os registros!');
        }
    }
    
    public function deletaProduto($params){
        try {
            $query = "DELETE FROM ITEMCOLE WHERE CDFILIAL = :P_CDFILIAL AND NRCOLEPRECO = :P_NRCOLEPRECO AND CDPRODUTO = :P_CDPRODUTO AND NRORG = :P_NRORG AND (CDREGIAO = :P_CDREGIAO OR :P_CDREGIAO = 'T') AND (CDFORNECED = :P_CDFORNECED OR :P_CDFORNECED = 'T') AND (CDMARCA = :P_CDMARCA OR :P_CDMARCA = 'T')";
            $params = [
                "P_CDFILIAL"    => $params['CDFILIAL'],
                "P_NRCOLEPRECO" => $params['NRCOLEPRECO'],    
                "P_CDREGIAO"    => $params['CDREGIAO'],
                "P_CDFORNECED"  => $params['CDFORNECED'], 
                "P_CDPRODUTO"   => $params['CDPRODUTO'], 
                "P_CDMARCA"     => $params['CDMARCA'], 
                'P_NRORG'       => $this->environment->getNrorg()
            ];
            $this->supUtilities->crud($query, $params);
        }
        catch (Exception $e) {
            throw new \Exception('Erro ao excluir os registros!');
        }
    }
    
    public function deletaPrazoPgt($params){
        try {
            $query = "DELETE FROM PZOPGTITMCOL WHERE PZOPGTITMCOL.CDREGIAO = :P_CDREGIAO AND PZOPGTITMCOL.CDFILIAL = :P_CDFILIAL AND PZOPGTITMCOL.NRCOLEPRECO = :P_NRCOLEPRECO AND PZOPGTITMCOL.CDFORNECED = :P_CDFORNECED AND PZOPGTITMCOL.CDPRODUTO = :P_CDPRODUTO AND PZOPGTITMCOL.CDMARCA = :P_CDMARCA AND PZOPGTITMCOL.NRORG = :P_NRORG";
            $params = [
                "P_CDREGIAO"    => $params['CDREGIAO'],
                "P_CDFILIAL"    => $params['CDFILIAL'],
                "P_NRCOLEPRECO" => $params['NRCOLEPRECO'],    
                "P_CDFORNECED"  => $params['CDFORNECED'], 
                "P_CDPRODUTO"   => $params['CDPRODUTO'], 
                "P_CDMARCA"     => $params['CDMARCA'], 
                'P_NRORG'       => $this->environment->getNrorg()
            ];
            $this->supUtilities->crud($query, $params);
        }
        catch (Exception $e) {
            throw new \Exception('Erro ao excluir os registros!');
        }
    }
    
    public function deletaPrazoItemCol($params){
        try {
            $query = "DELETE PZOPGTITMCOL WHERE CDFILIAL = :P_CDFILIAL AND NRCOLEPRECO = :P_NRCOLEPRECO AND NRORG = :P_NRORG";
            $params = [
                "P_CDFILIAL"    => $params['CDFILIAL'],
                "P_NRCOLEPRECO" => $params['NRCOLEPRECO'],   
                'P_NRORG'       => $this->environment->getNrorg()
            ];
            $this->supUtilities->crud($query, $params);
        }
        catch (Exception $e) {
            throw new \Exception('Erro ao excluir os registros!');
        }
    }
    
    public function deleteFornecedorRegister($params){
        try {
            $query = "DELETE FROM PZOPGTITMCOL WHERE CDFILIAL = :P_CDFILIAL AND NRCOLEPRECO = :P_NRCOLEPRECO AND CDFORNECED = :P_CDFORNECED AND CDPRODUTO = :P_CDPRODUTO AND CDMARCA  = :P_CDMARCA AND CDREGIAO = :P_CDREGIAO AND NRORG = :P_NRORG";
            $params = [
                "P_CDFILIAL"    => $params['CDFILIAL'],
                "P_NRCOLEPRECO" => $params['NRCOLEPRECO'],    
                "P_CDPRODUTO"   => $params['CDPRODUTO'],    
                "P_CDMARCA"     => $params['CDMARCA'],    
                "P_CDFORNECED"  => $params['CDFORNECED'],    
                "P_CDREGIAO"    => $params['CDREGIAO'],    
                'P_NRORG'       => $this->environment->getNrorg()
            ];
            $this->supUtilities->crud($query, $params);
        }
        catch (Exception $e) {
            throw new \Exception('Erro ao excluir os registros!');
        }
    }
    
    /***********************
     *      PROCEDURES     *
     * *********************/
    
    public function procIncluiItemcolTodos($row, $NRCOLEPREC){
        try{
            $procedure = new StoredProcedure($this->connection, 'INCLUI_ITEMCOL_TODOS');
            $procedure->addParam(new Param('P_CDFILIAL',    Param::PARAM_INPUT, $row['CDFILIAL'],    Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDFILIPROC',  Param::PARAM_INPUT, $row['CDFILIPROC'],  Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRCOLEPRECO', Param::PARAM_INPUT, $NRCOLEPREC,         Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_DTINI',       Param::PARAM_INPUT, $row["DTINIVALCOL"], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_DTFIN',       Param::PARAM_INPUT, $row["DTFINVALCOL"], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_PRODINI',     Param::PARAM_INPUT, $row["CDPRODINI"],   Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_PRODFIN',     Param::PARAM_INPUT, $row["CDPRODFIN"],   Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDREGIAO',    Param::PARAM_INPUT, $row["CDREGIAO"],    Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDFORNECED',  Param::PARAM_INPUT, $row["CDFORNECED"],  Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDMARCA',     Param::PARAM_INPUT, $row["CDMARCA"],     Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CONSIST',     Param::PARAM_INPUT, $row["CONSIST"],     Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_IDULTPREC',   Param::PARAM_INPUT, $row["IDULTPREC"],   Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDOPERINCL',  Param::PARAM_INPUT, $row["CDOPERINCL"],  Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_ROWSAFFECT',  Param::PARAM_OUTPUT, NULL,               Param::PARAM_TYPE_STR));
            return $procedure->execute();
        }catch(\Exception $e){
            return $e;
        }
    }
    
    public function procIncluiItemcolAComprar($row, $NRCOLEPREC){
        try{
            $procedure = new StoredProcedure($this->connection, 'INCLUI_ITEMCOL_ACOMPRAR');
            $procedure->addParam(new Param('P_CDFILIAL',    Param::PARAM_INPUT, $row['CDFILIAL'],    Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDFILIPROC',  Param::PARAM_INPUT, $row['CDFILIPROC'],  Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRCOLEPRECO', Param::PARAM_INPUT, $NRCOLEPREC,         Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_DTINI',       Param::PARAM_INPUT, $row["DTINIVALCOL"], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_DTFIN',       Param::PARAM_INPUT, $row["DTFINVALCOL"], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_PRODINI',     Param::PARAM_INPUT, $row["CDPRODINI"],   Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_PRODFIN',     Param::PARAM_INPUT, $row["CDPRODFIN"],   Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDREGIAO',    Param::PARAM_INPUT, $row["CDREGIAO"],    Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDFORNECED',  Param::PARAM_INPUT, $row["CDFORNECED"],  Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDMARCA',     Param::PARAM_INPUT, $row["CDMARCA"],     Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CONSIST',     Param::PARAM_INPUT, $row["CONSIST"],     Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_IDULTPREC',   Param::PARAM_INPUT, $row["IDULTPREC"],   Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDOPERINCL',  Param::PARAM_INPUT, $row["CDOPERINCL"],  Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_ROWSAFFECT',  Param::PARAM_OUTPUT, NULL,               Param::PARAM_TYPE_STR));
            return $procedure->execute();
        }catch(\Exception $e){
            return $e;
        }
    }
    
    public function procIncluiItemcolProdCompra($row, $NRCOLEPREC){
        try{
            $procedure = new StoredProcedure($this->connection, 'INCLUI_ITEMCOL_PRODCOMPRA');
            $procedure->addParam(new Param('P_CDFILIAL',    Param::PARAM_INPUT, $row['CDFILIAL'],    Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDFILIPROC',  Param::PARAM_INPUT, $row['CDFILIPROC'],  Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRCOLEPRECO', Param::PARAM_INPUT, $NRCOLEPREC,         Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_DTINI',       Param::PARAM_INPUT, $row['DTINIVALCOL'], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_DTFIN',       Param::PARAM_INPUT, $row['DTFINVALCOL'], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_PRODINI',     Param::PARAM_INPUT, $row['CDPRODINI'],   Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_PRODFIN',     Param::PARAM_INPUT, $row['CDPRODFIN'],   Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDREGIAO',    Param::PARAM_INPUT, $row['CDREGIAO'],    Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDFORNECED',  Param::PARAM_INPUT, $row['CDFORNECED'],  Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDMARCA',     Param::PARAM_INPUT, $row['CDMARCA'],     Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CONSIST',     Param::PARAM_INPUT, $row['CONSIST'],     Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_IDULTPREC',   Param::PARAM_INPUT, $row['IDULTPREC'],   Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDOPERINCL',  Param::PARAM_INPUT, $row['CDOPERINCL'],  Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_ROWSAFFECT',  Param::PARAM_OUTPUT, NULL,               Param::PARAM_TYPE_STR));
            return $procedure->execute();
        }catch(\Exception $e){
            return $e;
        }
    }
    
    public function procCargDescSolCot($row, $NRCOLEPREC){
        try{
            $procedure = new StoredProcedure($this->connection, 'CARG_DESCSOL_COT');
            $procedure->addParam(new Param('P_CDFILIAL',    Param::PARAM_INPUT, $row['CDFILIAL'],    Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRCOLEPRECO', Param::PARAM_INPUT, $NRCOLEPREC,         Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_DTINI',       Param::PARAM_INPUT, $row['DTINIVALCOL'], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_DTFIN',       Param::PARAM_INPUT, $row['DTFINVALCOL'], Param::PARAM_TYPE_STR));
            $procedure->execute();
        }catch(\Exception $e){
            return $e;
        }
    }
    
    public function procImpostoItemcol($params){
        try{
            $procedure = new StoredProcedure($this->connection, 'IMPOSTO_ITEMCOL');
            $procedure->addParam(new Param('P_EMPRESA',    Param::PARAM_INPUT, $params['CDEMPRESA'],   Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_INSCESTA',   Param::PARAM_INPUT, $params['CDINSCESTA'],  Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_FORNECEDOR', Param::PARAM_INPUT, $params['CDFORNECED'],  Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_FILCENT',    Param::PARAM_INPUT, $params['CDFILCENT'],   Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_LANCCOLP',   Param::PARAM_INPUT, $params['NRCOLEPRECO'], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDFORNECED', Param::PARAM_INPUT, $params['CDFORNECED'],  Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDREGIAO',   Param::PARAM_INPUT, $params['CDREGIAO'],    Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDPRODUTO',  Param::PARAM_INPUT, $params['CDPRODUTO'],   Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDMARCA',    Param::PARAM_INPUT, $params['CDMARCA'],     Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_INIVALID',   Param::PARAM_INPUT, $params['DTCOLEPREC'],  Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_TRANSACAO',  Param::PARAM_INPUT, null,                   Param::PARAM_TYPE_STR));
            $procedure->execute();
        }catch(\Exception $e){
            return $e;
        }
    }
    
    public function procIncFornCota($params){
        try{
            $procedure = new StoredProcedure($this->connection, 'INC_FORN_COTA');
            $procedure->addParam(new Param('P_CDFILIAL',    Param::PARAM_INPUT, $params['CDFILIAL'],    Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRCOLEPRECO', Param::PARAM_INPUT, $params['NRCOLEPRECO'], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDFORNECED',  Param::PARAM_INPUT, $params['CDFORNECED'],  Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDOPERSIS',   Param::PARAM_INPUT, $params['CDOPERINCL'],  Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_TREGIAO',     Param::PARAM_INPUT, $params['TREGIAO'],     Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDFORNCARG',  Param::PARAM_INPUT, $params['CDFORNCARG'],  Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDREGCARG',   Param::PARAM_INPUT, $params['CDREGCARG'],   Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_IDERRO',      Param::PARAM_OUTPUT, NULL,                  Param::PARAM_TYPE_STR));
            return $procedure->execute();
        }catch(\Exception $e){
            return $e;
        }
    }
    
    public function procItemcoleAtuCustosup($params){
        try{
            $procedure = new StoredProcedure($this->connection, 'ITEMCOLE_ATU_CUSTOSUP');
            $procedure->addParam(new Param('P_CDFILIAL',     Param::PARAM_INPUT, $params['CDFILIAL'],     Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRCOLEPRECO',  Param::PARAM_INPUT, $params['NRCOLEPRECO'],  Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDMARCA',      Param::PARAM_INPUT, $params['CDMARCA'],      Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDREGIAO',     Param::PARAM_INPUT, $params['CDREGIAO'],     Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDPRODUTO',    Param::PARAM_INPUT, $params['CDPRODUTO'],    Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRSEQPRELIST', Param::PARAM_INPUT, $params['NRSEQPRELIST'], Param::PARAM_TYPE_STR));
            $procedure->execute();
        }catch(\Exception $e){
            return $e;
        }
    }
    
    public function procPrecosToRegiao($params){
        try{
            $procedure = new StoredProcedure($this->connection, 'PRECOS_TO_REGIAO');
            $procedure->addParam(new Param('P_CDFILIAL',     Param::PARAM_INPUT, $params['CDFILIAL'],     Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDFILIPROC',   Param::PARAM_INPUT, $params['CDFILIAL'],   Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRCOLEPRECO',  Param::PARAM_INPUT, $params['NRCOLEPRECO'],  Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDREGIAO',     Param::PARAM_INPUT, $params['CDREGIAO'],     Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDFORNECED',   Param::PARAM_INPUT, $params['CDFORNECED'],   Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_TFORN',        Param::PARAM_INPUT, '',                      Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDPRODUTO',    Param::PARAM_INPUT, $params['CDPRODUTO'],    Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDMARCA',      Param::PARAM_INPUT, $params['CDMARCA'],      Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_IDATUREG',     Param::PARAM_INPUT, $params['P_IDATUREG'],   Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_IDATUFORN',    Param::PARAM_INPUT, $params['P_IDATUFORN'],  Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NEW_CDREGIAO', Param::PARAM_INPUT, $params['NEW_CDREGIAO'], Param::PARAM_TYPE_STR));
            $procedure->execute();
        }catch(\Exception $e){
            throw $e;
        }
    }
    
    public function procPrecosToForn($params){
        try{
            $procedure = new StoredProcedure($this->connection, 'PRECOS_TO_FORN');
            $procedure->addParam(new Param('P_CDFILIAL',     Param::PARAM_INPUT, $params['CDFILIAL'],     Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDFILIPROC',   Param::PARAM_INPUT, $params['CDFILIAL'],   Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRCOLEPRECO',  Param::PARAM_INPUT, $params['NRCOLEPRECO'],  Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDREGIAO',     Param::PARAM_INPUT, $params['CDREGIAO'],     Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDFORNECED',   Param::PARAM_INPUT, $params['CDFORNECED'],   Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDPRODUTO',    Param::PARAM_INPUT, $params['CDPRODUTO'],    Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDMARCA',      Param::PARAM_INPUT, $params['CDMARCA'],      Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_IDATUREG',     Param::PARAM_INPUT, $params['P_IDATUREG'],   Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_IDATUFORN',    Param::PARAM_INPUT, $params['P_IDATUFORN'],  Param::PARAM_TYPE_STR));
            $procedure->execute();
        }catch(\Exception $e){
            throw $e;
        }
    }
    
    public function procDuplicaCotacao($params){
        try{
            $procedure = new StoredProcedure($this->connection, 'DUPLICA_COTACAO');
            $procedure->addParam(new Param('P_CDFILIAL',    Param::PARAM_INPUT,  $params['CDFILIAL'],    Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDFILIPROC',  Param::PARAM_INPUT,  $params['CDFILIPROC'],  Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRCOLEPRECO', Param::PARAM_INPUT,  $params['NRCOLEPRECO'], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRSEMAINIC',  Param::PARAM_INPUT,  $params['NRSEMAINIC'],  Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRSEMAFINAL', Param::PARAM_INPUT,  $params['NRSEMAFINAL'], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_DTNOVINIVAL', Param::PARAM_INPUT,  $params['DTNOVINIVAL'], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_DTNOVFINVAL', Param::PARAM_INPUT,  $params['DTNOVFINVAL'], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_DTINIVALCOL', Param::PARAM_INPUT,  $params['DTINIVALCOL'], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_DTFINVALCOL', Param::PARAM_INPUT,  $params['DTFINVALCOL'], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_DTCOLEPREC',  Param::PARAM_INPUT,  $params['DTCOLEPREC'],  Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_REAJUSTE',    Param::PARAM_INPUT,  (float)$params['REAJUSTE'],    Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRCOLEDUP',   Param::PARAM_INPUT,  $params['NRCOLEDUP'],   Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_IDDUPLICA',   Param::PARAM_INPUT,  $params['IDDUPLICA'],   Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDOPERADOR',  Param::PARAM_INPUT,  $params['CDOPERADOR'],  Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_TCDOPERADOR', Param::PARAM_INPUT,  $params['TCDOPERADOR'], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDOPERINCL',  Param::PARAM_INPUT,  $params['CDOPERINCL'],  Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDREGIAO',    Param::PARAM_INPUT,  $params['CDREGIAO'],    Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_COTA_N_DUP',  Param::PARAM_OUTPUT, NULL,                   Param::PARAM_TYPE_STR));
            return $procedure->execute();
        }catch(\Exception $e){
            return $e;
        }
    }
    
    public function procMelhorPrecoAuto($params){
        try{
            $procedure = new StoredProcedure($this->connection, 'MELHOR_PRECO_AUTO');
            $procedure->addParam(new Param('P_CDFILIAL',    Param::PARAM_INPUT,  $params['CDFILIAL'],    Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRCOLEPRECO', Param::PARAM_INPUT,  $params['NRCOLEPRECO'], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_DTINI',       Param::PARAM_INPUT,  $params['DTINIVALCOL'], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_DTFIN',       Param::PARAM_INPUT,  $params['DTFINVALCOL'], Param::PARAM_TYPE_STR));
            return $procedure->execute();
        }catch(\Exception $e){
            return $e;
        }
    }
    
    public function procPrecoEconAf($params){
        try{
            $procedure = new StoredProcedure($this->connection, 'PRECO_ECON_AF');
            $procedure->addParam(new Param('P_CDFILIAL',    Param::PARAM_INPUT,  $params['CDFILIAL'],    Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDFILDEST',   Param::PARAM_INPUT,  $params['CDFILDEST'],   Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDFILLANC',   Param::PARAM_INPUT,  $params['CDFILLANC'],   Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRAUTOFORN',  Param::PARAM_INPUT,  $params['NRAUTOFORN'],  Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDPRODUTO',   Param::PARAM_INPUT,  $params['CDPRODUTO'],   Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDMARCA',     Param::PARAM_INPUT,  $params['CDMARCA'],     Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_DTPROGENTAF', Param::PARAM_INPUT,  $params['DTPROGENTAF'], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_VRPRECECO',   Param::PARAM_OUTPUT, NULL,                   Param::PARAM_TYPE_STR, 10));
            return $procedure->execute();
        }catch(\Exception $e){
            return $e;
        }
    }
    
    public function procAtualizaItmapur($params){
        try{
            $procedure = new StoredProcedure($this->connection, 'ATUALIZA_ITMAPUR');
            $procedure->addParam(new Param('P_CDCENTCMP',   Param::PARAM_INPUT,  $params['CDFILCENT'],  Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRSEQAPUR',   Param::PARAM_INPUT,  $params['NRSEQAPUR'],  Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDPRODUTO',   Param::PARAM_INPUT,  $params['CDPRODUTO'],  Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDMARCA',     Param::PARAM_INPUT,  $params['CDMARCA'],    Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDFORNECED',  Param::PARAM_INPUT,  $params['CDFORNECED'], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRCOLEPRECO', Param::PARAM_INPUT,  $params['NRCOLEPREC'], Param::PARAM_TYPE_STR));
            $procedure->execute();
        }catch(\Exception $e){
            return $e;
        }
    }
    
    public function verifyProdOnSupplyTek($row, $type = "single")
    {
        if ($type == "single") {
            $conditions = "AND PS.CDPRODUTO = :P_CDPRODUTO
                           AND PS.CDMARCA = :P_CDMARCA";
                           
            $params = [
                'P_CDPRODUTO' => $row['CDPRODUTO'],
                'P_CDMARCA' => $row['CDMARCA'],
            ];
        } else {
            $conditions = "AND ( PR.CDARVPROD BETWEEN :P_CDPRODINI AND :P_CDPRODFIN )";
            
            $params = [
                'P_CDPRODINI' => $row['CDPRODINI'],
                'P_CDPRODFIN' => $row['CDPRODFIN'],
            ];
        }
        $sql = "SELECT
                    CS.NRCARTSUPPLY,
                    CS.CDFILIAL,
                    PR.NMPRODUTO,
                    M.NMMARCA, 
                    CS.CDOPERADOR
                FROM
                    PRODCART PC,
                    PRODSUPPLY PS,
                    CARTSUPPLY CS,
                    PRODUTO PR,
                    MARCA M
                WHERE
                    PC.NRCARTSUPPLY = CS.NRCARTSUPPLY
                    AND PS.CDCATEGORIA = PC.CDCATEGORIA
                    AND PS.CDSUBCATEGORIA = PC.CDSUBCATEGORIA
                    AND PS.CDPRODUTO = PC.CDPRODUTO
                    AND PS.CDFORNECED = CS.CDFORNECED
                    AND PR.CDPRODUTO = PS.CDPRODUTO
                    AND M.CDMARCA = PS.CDMARCA
                    " . $conditions . "
                    AND PS.NRCOLEPRECO = :P_NRCOLEPRECO
                    AND PS.CDFORNECED = :P_CDFORNECED
                    AND PS.CDFILCOT = :P_CDFILIAL
                    AND PS.CDREGIAO = :P_CDREGIAO
                    AND PS.NRORG = :P_NRORG
                    AND CS.IDSITUCOMPRA = 'A'";
        
        $params['P_NRCOLEPRECO'] = $row['NRCOLEPRECO'];
        $params['P_CDFORNECED']  = $row['CDFORNECED'];
        $params['P_CDFILIAL']    = $row['CDFILIAL'];
        $params['P_CDREGIAO']    = $row['CDREGIAO'];
        $params['P_NRORG']       = $this->environment->getNrorg();

        return $this->supUtilities->crud($sql, $params);
    }
    
    public function insertNotificationSupplyTek($params)
    {
        try {
            $sql = "INSERT INTO NOTIFICSUPPLY
                    (
                        NRNOTIFICSUPPLY,
                        CDFILIAL,
                        CDOPERADOR,
                        NMNOTIFICACAO,
                        DSTEXTNOTIFIC,
                        IDNOTIFICLIDO,
                        NRORG,
                        DTINCLUSAO,
                        IDATIVO
                    )
                    VALUES
                    (
                        :P_NRNOTIFICSUPPLY,
                        :P_CDFILIAL,
                        :P_CDOPERADOR,
                        :P_NMNOTIFICACAO,
                        :P_DSTEXTNOTIFIC,
                        'N',
                        :P_NRORG,
                        SYSDATE,
                        'S'
                    )";
            
            
            $param = [
                "P_NRNOTIFICSUPPLY" => $params['NRNOTIFICSUPPLY'],
                "P_CDFILIAL" => $params['CDFILIALSUPPLYTEK'],
                "P_CDOPERADOR" => $params['CDOPERADOR'],
                "P_NMNOTIFICACAO" => $params['NMNOTIFICACAO'],
                "P_DSTEXTNOTIFIC" => $params['DSTEXTNOTIFIC'],
                'P_NRORG' => $this->environment->getNrorg()
            ];
            return $this->supUtilities->crud($sql, $param);
        }
        catch (Exception $e) {
            throw new \Exception('Erro ao inserir notificação para o SupplyTek!');
        }
    }
    
    public function centralCompras($row) {
        $query = "SELECT NVL(CDFILCENTCMP, CDFILIAL) CDFILCENTCMP FROM FILIAL WHERE CDFILIAL = :P_CDFILIAL";
        
        $param = [
            'P_CDFILIAL' => $row['CDFILIAL']
        ];
        
        return $this->supUtilities->crud($query, $param);
    }
}