<?php
namespace Teknisa\SUP\Modules\Fornecimento\Modules\Relatorios\Controllers;

use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Doctrine\ORM\EntityManager;

class sup02200CurvaAbcProdutosFornecedorComprasController extends \Zeedhi\Framework\Controller\Report 
{
    protected $birtStrategy;
    protected $environment;
    protected $entityManager;
    
    public function __construct($reportService, $birtStrategy, $environment, $connection, $entityManager) 
    {
        parent::__construct($reportService);
        $this->birtStrategy  = $birtStrategy;
        $this->environment   = $environment;
        $this->connection    = $connection;
        $this->entityManager = $entityManager;
    }
    
    public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response)
    {
        try {
            $row = $request->getRow();

            $row['TCDGRUPFILI']     = $row['CDGRUPFILI']    ? '' : 'T';
            $row['TCDREGIAO']       = $row['CDREGIAO']      ? '' : 'T';
            $row['TCDFILIAL']       = $row['CDFILIAL']      ? '' : 'T';
            $row['TCDSUBGRUPO']     = $row['CDSUBGRUPO']    ? '' : 'T';
            $row['TCDFORNECED']     = $row['CDFORNECED']    ? '' : 'T';
            $row['TCDOPERADOR']     = $row['CDOPERADOR']    ? '' : 'T';
            $row['TIDBAIXAAF']      = $row['IDBAIXAAF']     ? '' : 'T';
            
            $row['TIDAFEXPIRADA']   = $row['IDAFEXPIRADA'] == 'S' ? 'T' : '';
            $row['IDAFEXPIRADA']    = $row['IDAFEXPIRADA'] == 'S' ? ''  : $row['IDAFEXPIRADA'];
            
            $row['IDFILIATIVA']     = $row['IDFILIATIVA'];
            $row['TIDFILIATIVA']    = $row['IDFILIATIVA'];
            $row['EXIBE_PROD']      = $row['IDEXIBPRODUTO'];
            
            $row['IDCUSTO']         = '';
            
            $row['IDEXIBGRUPFILI']  = '';
            
            $row['CDFILOGADA']      = $this->environment->getCdFilial();
            $row['OPERADORLOG']     = $this->environment->getCdoperador();
            $row['NRORG']           = $this->environment->getNrorg();
            
            $this->birtStrategy->setReportFormat(strtolower($row['QR2TYPE']));
            $this->stNomeRel = $this->getReportName($row);
            $this->reportMapping[$this->stNomeRel] =
                array (
                    'strategy' => $row['STRATEGY'],
                    'parameterMapping' =>
                        array (
                            "RELATORIO"         => "P_RELATORIO",
                            "IDTIPOREL"         => "P_DADO",
                            "CDGRUPFILI"        => "P_CDGRUPFILI",
                            "TCDGRUPFILI"       => "P_TCDGRUPFILI",
                            "IDEXIBGRUPFILI"    => "P_EXIBE_GRUPFILI",
                            "CDREGIAO"          => "P_CDREGIAO",
                            "TCDREGIAO"         => "P_TCDREGIAO",
                            "IDEXIBREG"         => "P_EXIBE_REGIAO",
                            "CDFILIAL"          => "P_CDFILIAL",
                            "TCDFILIAL"         => "P_TCDFILIAL",
                            "IDEXIFILI"         => "P_EXIBE_FILIAL",
                            "DTENTRINI"         => "P_DTENTRINI",
                            "DTENTRFIN"         => "P_DTENTRFIN",
                            "TDTENTREGA"        => "P_TDTENTREGA",
                            "DTGERAINI"         => "P_DTGERAINI",
                            "DTGERAFIN"         => "P_DTGERAFIN",
                            "TDTENTRADA"        => "P_TDTGERACAO",
                            "CDNVPRODUTO"       => "P_CDNVPRODUTO",
                            "CDSUBGRUPO"        => "P_SUBGRUPO",
                            "TCDSUBGRUPO"       => "P_TSUBGRUPO",
                            "CDFORNECED"        => "P_CDFORNECED",
                            "TCDFORNECED"       => "P_TCDFORNECED",
                            "CDOPERADOR"        => "P_CDOPERADOR",
                            "TCDOPERADOR"       => "P_TCDOPERADOR",
                            "CDPRODINI"         => "P_CDPRODINI",
                            "CDPRODFIN"         => "P_CDPRODFIN",
                            "IDBAIXAAF"         => "P_IDBAIXAAF",
                            "TIDBAIXAAF"        => "P_TIDBAIXAAF",
                            "IDAFEXPIRADA"      => "P_IDAFEXPIRADA",
                            "TIDAFEXPIRADA"     => "P_TIDAFEXPIRADA",
                            "CONSIDERA"         => "P_CONSISTE_BAIXA",
                            "CONSIDERA"         => "P_VALOR_EFETIVADO",
                            "IDFILIATIVA"       => "P_IDFILIATIVA",
                            "TIDFILIATIVA"      => "P_TIDFILIATIVA",
                            "CDFILOGADA"        => "P_FILIAL_LOGADA",
                            "EXIBE_PROD"        => "P_EXIBE_PROD",
                            "IDCUSTO"           => "P_CUSTO",
                            "OPERADORLOG"       => "P_OPERADORLOG",
                            "NRORG"             => "P_NRORG"
                        ),
                );
        } catch (Exception $e) {
            $response->setError(Error::factoryFromException($e, $e->getMessage(), $e->getCode()));
        }
    }
    
    public function getSubGrupProd(DTO\Request\Row $request, DTO\Response $response)
    {
        $row = $request->getRow();
        
        $params = array(
    		"P_CDFILCENTCMP" => $row['CDFILCENTCMP']
    	);
    	
    	$sql = "SELECT CDGRUPPROD FROM PARASUPRI WHERE CDFILIAL =:P_CDFILCENTCMP";
    	$query = $this->connection->prepare($sql);
    	$query->execute($params);
    	$result = $query->fetch();
    	
    	if (empty($result)) {
    	   $result = [ 'CDGRUPPROD' => '' ];
    	}
    	
    	$response->addDataSet(new DataSet('data', [$result]));
    }
}
